@@ -120,10 +120,10 @@ class objInst:
         self.embeddedComponent = 0
 
 class LdtpClientContext:
-    def __init__ (self, localeLang, windowName, appHandleaccessible, appmap, request, response, guiHandle):
+    def __init__ (self, localeLang, windowName, appHandle, appmap, request, response, guiHandle):
         self.localeLang = localeLang
         self.windowName = windowName
-        self.appHandle  = appHandleaccessible
+        self.appHandle  = appHandle
         self.appmap     = appmap
         self.request    = request
         self.response   = response
@@ -183,14 +183,18 @@ class objInfo:
 class widget:
     def __init__ (self, widgetType, *args):
         self.widgetType = widgetType
-        if len (args) == 3:
+        if len (args) == 5:
             self.label = args [0]
             self.parent = args [1]
             self.index = args [2]
+            self.key   = args [3]
+            self.hierarchy = args [4]
         else:
             self.label = ''
             self.parent = args [0]
             self.index = args [1]
+            self.key   = args [2]
+            self.hierarchy = args [3]
 
 class window (widget):
     def __init__ (self, *args):
@@ -211,6 +215,10 @@ class keyboard:
                       12 : '#', 13 : '$', 14 : '%', 15 : '^', 16 : '&', 17 : '*', 18 : '(', \
                       19 : ')', 20 : '_', 21 : '+', 34 : '{', 35 : '}', 47 : ':', 48 : '\"', \
                       49 : '~', 51 : '|', 59 : '<', 60 : '>', 61 : '?'}
+
+    # PrtScrn: http://gentoo-wiki.com/TIP_Make_a_Screenshot_with_PrintScreen_Key
+    # xmodmap -pke | grep -i print
+
     nonPrintKeySynth = {9 : '<escape>', 22 : '<backspace>',
                         37 : '<ctrl>', 115 : '<windowskey>',
                         23 : '<tab>', 36 : '<return>',
@@ -225,168 +233,223 @@ class keyboard:
                         77 : '<numlock>', 78 : '<scrolllock>',
                         67 : '<F1>', 68 : '<F2>', 69 : '<F3>', 70 : '<F4>',
                         71 : '<F5>', 72 : '<F6>', 73 : '<F7>', 74 : '<F8>',
-                        75 : '<F9>', 76 : '<F10>', 95 : '<F11>', 96 : '<F12>'}
+                        75 : '<F9>', 76 : '<F10>', 95 : '<F11>', 96 : '<F12>',
+                        111 : "prtscrn"}
 
 class appmap:
-    def __init__ (self, objectInst,  registry):
-        self.hashmap = None
-        self.hierarchy = ''
-        self.registry    = registry
+    def __init__ (self, objectInst, registry):
+        self.hashmap    = None
+        self.hierarchy  = ''
+        self.nodeIndex  = 0
+        self.objTimeout = 0
+        self.registry   = registry
         self.objectInst = objectInst
-        self.applicationmap = window ('list_of_windows', None, None).children
+        self.lastNewContext      = None
+        self.lastExistingContext = None
+        self.applicationmap = window ('list_of_windows', None, None, None, None).children
 
     def getObjectInfo (self, accessible, parentName):
         role = accessible.getRole ()
         objectInfo = None
         if role == Accessibility.ROLE_PAGE_TAB:
             objectInfo = objInfo ('ptab', 'page_tab', self.objectInst.pageTab, parentName)
+            self.objectInst.instIndex = self.objectInst.pageTab
             self.objectInst.pageTab += 1
         elif role == Accessibility.ROLE_PAGE_TAB_LIST:
             objectInfo = objInfo ('ptl', 'page_tab_list', self.objectInst.pageTabList, parentName)
+            self.objectInst.instIndex = self.objectInst.pageTabList
             self.objectInst.pageTabList += 1
         elif role == Accessibility.ROLE_MENU:
             objectInfo = objInfo ('mnu', 'menu', self.objectInst.menu, parentName)
+            self.objectInst.instIndex = self.objectInst.menu
             self.objectInst.menu += 1
         elif role == Accessibility.ROLE_MENU_ITEM:
             objectInfo = objInfo ('mnu', 'menu_item', self.objectInst.menu, parentName)
+            self.objectInst.instIndex = self.objectInst.menu
             self.objectInst.menu += 1
         elif role == Accessibility.ROLE_CHECK_MENU_ITEM:
             objectInfo = objInfo ('mnu', 'check_menu_item', self.objectInst.menu, parentName)
+            self.objectInst.instIndex = self.objectInst.menu
             self.objectInst.menu += 1
         elif role == Accessibility.ROLE_RADIO_MENU_ITEM:
             objectInfo = objInfo ('mnu', 'radio_menu_item', self.objectInst.menu, parentName)
+            self.objectInst.instIndex = self.objectInst.menu
             self.objectInst.menu += 1
         elif role == Accessibility.ROLE_CHECK_BOX:
             objectInfo = objInfo ('chk', 'check_box', self.objectInst.checkBox, parentName)
+            self.objectInst.instIndex = self.objectInst.checkBox
             self.objectInst.checkBox += 1
         elif role == Accessibility.ROLE_RADIO_BUTTON:
             objectInfo = objInfo ('rbtn', 'radio_button', self.objectInst.radioButton, parentName)
+            self.objectInst.instIndex = self.objectInst.radioButton
             self.objectInst.radioButton += 1
         elif role == Accessibility.ROLE_SPIN_BUTTON:
             objectInfo = objInfo ('sbtn', 'spin_button', self.objectInst.spinButton, parentName)
+            self.objectInst.instIndex = self.objectInst.spinButton
             self.objectInst.spinButton += 1
         elif role == Accessibility.ROLE_PUSH_BUTTON:
             objectInfo = objInfo ('btn', 'push_button', self.objectInst.button, parentName)
+            self.objectInst.instIndex = self.objectInst.button
             self.objectInst.button += 1
         elif role == Accessibility.ROLE_TOGGLE_BUTTON:
             objectInfo = objInfo ('tbtn', 'toggle_button', self.objectInst.tbutton, parentName)
+            self.objectInst.instIndex = self.objectInst.tbutton
             self.objectInst.tbutton += 1
         elif role == Accessibility.ROLE_TEXT:
             objectInfo = objInfo ('txt', 'text', self.objectInst.text, parentName)
+            self.objectInst.instIndex = self.objectInst.text
             self.objectInst.text += 1
         elif role == Accessibility.ROLE_PASSWORD_TEXT:
             objectInfo = objInfo ('txt', 'password_text', self.objectInst.text, parentName)
+            self.objectInst.instIndex = self.objectInst.text
             self.objectInst.text += 1
         elif role == Accessibility.ROLE_EDITBAR:
             objectInfo = objInfo ('txt', 'edit_bar', self.objectInst.text, parentName)
+            self.objectInst.instIndex = self.objectInst.text
             self.objectInst.text += 1
         elif role == Accessibility.ROLE_ENTRY:
             objectInfo = objInfo ('txt', 'entry', self.objectInst.text, parentName)
+            self.objectInst.instIndex = self.objectInst.text
             self.objectInst.text += 1
         elif role == Accessibility.ROLE_LIST:
             objectInfo = objInfo ('lst', 'list', self.objectInst.lst, parentName)
+            self.objectInst.instIndex = self.objectInst.lst
             self.objectInst.lst += 1
         elif role == Accessibility.ROLE_TABLE:
             objectInfo = objInfo ('tbl', 'table', self.objectInst.table, parentName)
+            self.objectInst.instIndex = self.objectInst.table
             self.objectInst.table += 1
         elif role == Accessibility.ROLE_TABLE_COLUMN_HEADER:
             objectInfo = objInfo ('tbl', 'table_column_header', self.objectInst.columnHeader, parentName)
+            self.objectInst.instIndex = self.objectInst.columnHeader
             self.objectInst.columnHeader += 1
         elif role == Accessibility.ROLE_TREE:
             objectInfo = objInfo ('tree', 'tree', self.objectInst.tree, parentName)
+            self.objectInst.instIndex = self.objectInst.tree
             self.objectInst.tree += 1
         elif role == Accessibility.ROLE_TREE_TABLE:
             objectInfo = objInfo ('ttbl', 'tree_table', self.objectInst.treeTable, parentName)
+            self.objectInst.instIndex = self.objectInst.treeTable
             self.objectInst.treeTable += 1
         elif role == Accessibility.ROLE_TABLE_CELL:
             objectInfo = objInfo ('tblc', 'table_cell', -1, parentName)
+            self.objectInst.instIndex = -1
         elif role == Accessibility.ROLE_FRAME:
             objectInfo = objInfo ('frm', 'frame', -1, parentName)
+            self.objectInst.instIndex = -1
         elif role == Accessibility.ROLE_DIALOG:
             objectInfo = objInfo ('dlg', 'dialog', self.objectInst.dialog, parentName)
+            self.objectInst.instIndex = self.objectInst.dialog
             self.objectInst.dialog += 1
         elif role == Accessibility.ROLE_WINDOW:
             objectInfo = objInfo ('dlg', 'dialog', self.objectInst.dialog, parentName)
+            self.objectInst.instIndex = self.objectInst.dialog
             self.objectInst.dialog += 1
         elif role == Accessibility.ROLE_FONT_CHOOSER:
             objectInfo = objInfo ('dlg', 'font_chooser', -1, parentName)
+            self.objectInst.instIndex = -1
         elif role == Accessibility.ROLE_FILE_CHOOSER:
             objectInfo = objInfo ('dlg', 'file_chooser', -1, parentName)
+            self.objectInst.instIndex = -1
         elif role == Accessibility.ROLE_ALERT:
             objectInfo = objInfo ('dlg', 'alert', self.objectInst.alert, parentName)
+            self.objectInst.instIndex = self.objectInst.alert
             self.objectInst.alert += 1
         elif role == Accessibility.ROLE_COMBO_BOX:
             objectInfo = objInfo ('cbo', 'combo_box', self.objectInst.comboBox, parentName)
+            self.objectInst.instIndex = self.objectInst.comboBox
             self.objectInst.comboBox += 1
         elif role == Accessibility.ROLE_LAYERED_PANE:
             objectInfo = objInfo ('pane', 'layered_pane', self.objectInst.layeredPane, parentName)
+            self.objectInst.instIndex = self.objectInst.layeredPane
             self.objectInst.layeredPane += 1
         elif role == Accessibility.ROLE_CALENDAR:
             objectInfo = objInfo ('calview', 'calendar_view', self.objectInst.calView, parentName)
+            self.objectInst.instIndex = self.objectInst.calView
             self.objectInst.calView += 1
         elif role == Accessibility.ROLE_PANEL:
             objectInfo = objInfo ('pnl', 'panel', self.objectInst.panel, parentName)
+            self.objectInst.instIndex = self.objectInst.panel
             self.objectInst.panel += 1
         elif role == Accessibility.ROLE_LABEL:
             objectInfo = objInfo ('lbl', 'label', self.objectInst.label, parentName)
+            self.objectInst.instIndex = self.objectInst.label
             self.objectInst.label += 1
         elif role == Accessibility.ROLE_ICON:
             objectInfo = objInfo ('ico', 'icon', self.objectInst.icon, parentName)
+            self.objectInst.instIndex = self.objectInst.icon
             self.objectInst.icon += 1
         elif role == Accessibility.ROLE_MENU_BAR:
             objectInfo = objInfo ('mbr', 'menu_bar', self.objectInst.menuBar, parentName)
+            self.objectInst.instIndex = self.objectInst.menuBar
             self.objectInst.menuBar += 1
         elif role == Accessibility.ROLE_SCROLL_BAR:
             objectInfo = objInfo ('scbr', 'scroll_bar', self.objectInst.scrollBar, parentName)
+            self.objectInst.instIndex = self.objectInst.scrollBar
             self.objectInst.scrollBar += 1
         elif role == Accessibility.ROLE_SCROLL_PANE:
             objectInfo = objInfo ('scpn', 'scroll_pane', self.objectInst.scrollPane, parentName)
+            self.objectInst.instIndex = self.objectInst.scrollPane
             self.objectInst.scrollPane += 1
         elif role == Accessibility.ROLE_STATUS_BAR:
             objectInfo = objInfo ('stat', 'status_bar', self.objectInst.statusBar, parentName)
+            self.objectInst.instIndex = self.objectInst.statusBar
             self.objectInst.statusBar += 1
         elif role == Accessibility.ROLE_SEPARATOR:
             objectInfo = objInfo ('spr', 'separator', self.objectInst.separator, parentName)
+            self.objectInst.instIndex = self.objectInst.separator
             self.objectInst.separator += 1
         elif role == Accessibility.ROLE_FILLER:
             objectInfo = objInfo ('flr', 'filler', self.objectInst.filler, parentName)
+            self.objectInst.instIndex = self.objectInst.filler
             self.objectInst.filler += 1
         elif role == Accessibility.ROLE_CANVAS:
             objectInfo = objInfo ('cnvs', 'canvas', self.objectInst.canvas, parentName)
+            self.objectInst.instIndex = self.objectInst.canvas
             self.objectInst.canvas += 1
         elif role == Accessibility.ROLE_SLIDER:
             objectInfo = objInfo ('sldr', 'slider', self.objectInst.slider, parentName)
+            self.objectInst.instIndex = self.objectInst.slider
             self.objectInst.slider += 1
         elif role == Accessibility.ROLE_SPLIT_PANE:
             objectInfo = objInfo ('splt', 'split_pane', self.objectInst.splitPane, parentName)
+            self.objectInst.instIndex = self.objectInst.splitPane
             self.objectInst.splitPane += 1
         elif role == Accessibility.ROLE_HTML_CONTAINER:
             objectInfo = objInfo ('html', 'html_container', self.objectInst.htmlContainer, parentName)
+            self.objectInst.instIndex = self.objectInst.htmlContainer
             self.objectInst.htmlContainer += 1
         elif role == Accessibility.ROLE_PROGRESS_BAR:
             objectInfo = objInfo ('pbr', 'progress_bar', self.objectInst.progressBar, parentName)
+            self.objectInst.instIndex = self.objectInst.progressBar
             self.objectInst.progressBar += 1
         elif role == Accessibility.ROLE_TOOL_BAR:
             objectInfo = objInfo ('tbar', 'tool_bar', self.objectInst.toolBar, parentName)
+            self.objectInst.instIndex = self.objectInst.toolBar
             self.objectInst.toolBar += 1
         elif role == Accessibility.ROLE_EMBEDDED:
             objectInfo = objInfo ('emb', 'embedded_component', self.objectInst.embeddedComponent, parentName)
+            self.objectInst.instIndex = self.objectInst.embeddedComponent
             self.objectInst.embeddedComponent += 1
         elif role == Accessibility.ROLE_APPLICATION:
             objectInfo = objInfo ('app', 'application', -1, parentName)
+            self.objectInst.instIndex = -1
         elif role == Accessibility.ROLE_EXTENDED:
             name = accessible.getRoleName ()
             if name == 'Calendar View':
                 objectInfo = objInfo ('cal', 'calendar_view', self.objectInst.calView, parentName)
+                self.objectInst.instIndex = self.objectInst.calView
                 self.objectInst.calView += 1
             elif name == 'Calendar Event':
                 objectInfo = objInfo ('cal', 'calendar_event', self.objectInst.calView, parentName)
+                self.objectInst.instIndex = self.objectInst.calView
                 self.objectInst.calView += 1
             else:
                 if name is not None and name != '':
                     name = re.sub (' ', '_', name)
                 # print 'extended', accessible.getRoleName (), accessible.getRole (), '*', accessible.name, '*'
                 objectInfo = objInfo ('ukn', name, self.objectInst.unknown, parentName)
+                self.objectInst.instIndex = self.objectInst.unknown
                 self.objectInst.unknown += 1
         else:
             # print 'notlisted', accessible.getRoleName (), accessible.getRole (), '*', accessible.name, '*'
@@ -396,6 +459,7 @@ class appmap:
             else:
                 name = re.sub (' ', '_', roleName)
             objectInfo = objInfo ('ukn', name, self.objectInst.unknown, parentName)
+            self.objectInst.instIndex = self.objectInst.unknown
             self.objectInst.unknown += 1
         return objectInfo
 
@@ -410,7 +474,9 @@ class appmap:
             num += 1
         return name + str (num)
 
-    def dumpTree (self, accessible, parentName = None,  forceReScan = False):
+    def doRemap (self, accessible, parentName = None, forceReScan = False, overloadedWindowName = False):
+        if accessible is None:
+            return
         role = accessible.getRole ()
         if role == Accessibility.ROLE_TABLE_CELL or \
                role == Accessibility.ROLE_SEPARATOR or \
@@ -428,70 +494,87 @@ class appmap:
         #if _ldtpDebug and _ldtpDebug == '2':
         #    print '%s* %s "%s": %s' % ('\t', roleName, name, description)
         objectInfo = self.getObjectInfo (accessible, parentName)
+        objHierarchyName = "%s#%d" % (objectInfo.prefix, objectInfo.instIndex)
         if objectInfo is not None and role != Accessibility.ROLE_APPLICATION:
             flag = False
             if role == Accessibility.ROLE_FRAME or role == Accessibility.ROLE_DIALOG or \
                    role == Accessibility.ROLE_ALERT or role == Accessibility.ROLE_FONT_CHOOSER or \
                    role == Accessibility.ROLE_FILE_CHOOSER or role == Accessibility.ROLE_WINDOW:
                 flag = True
-            if name != 'unnamed' and re.sub ("(?i) +", "", name) != '':
+            if name != 'unnamed' and re.sub ("(?i) +", "", name) != '' and overloadedWindowName is False:
                 strippedStr = escapeChars (name, False)
-                own_name = objectInfo.prefix + strippedStr
+                ownName = objectInfo.prefix + strippedStr
                 try:
-                    tmpName = own_name.encode ('utf-8')
+                    ownName = ownName.encode ('utf-8')
                 except UnicodeDecodeError:
-                    own_name = unicode (own_name, 'utf-8').encode ('utf-8')
+                    ownName = unicode (ownName, 'utf-8').encode ('utf-8')
                 if flag:
-                    if self.applicationmap != {} and own_name in self.applicationmap and forceReScan == False:
+                    if self.applicationmap != {} and ownName in self.applicationmap and forceReScan == False:
                         if _ldtpDebug:
-                            print 'has_key', own_name
+                            print 'has_key', ownName
                         # Since the window info is already in appmap, don't process further
                         return
-                    cur_win = window (objectInfo.objType, name, parentName, accessible.getIndexInParent ())
-                    if _ldtpDebug:
-                        print 'Window:',  own_name,  cur_win,  type (own_name),  type (cur_win),  own_name in self.applicationmap,  forceReScan
-                    if own_name in self.applicationmap and forceReScan:
-                        del self.applicationmap [own_name]
-                    if _ldtpDebug:
-                        print 'Window:',  own_name,  cur_win,  type (own_name),  type (cur_win),  own_name in self.applicationmap,  forceReScan
-                    self.applicationmap [own_name] = cur_win
-                    self.hashmap = cur_win.children
+                    curWindow = window (objectInfo.objType, name, parentName,
+                                        accessible.getIndexInParent (),
+                                        ownName, objHierarchyName)
+                    if ownName in self.applicationmap and forceReScan:
+                        del self.applicationmap [ownName]
+                    self.applicationmap [ownName] = curWindow
+                    self.hashmap = curWindow.children
                 #else:
-                #    if self.hashmap and own_name in self.hashmap:
-                #        own_name = self.get_non_conflicting_name (own_name)
+                #    if self.hashmap and ownName in self.hashmap:
+                #        ownName = self.get_non_conflicting_name (ownName)
                 #    if _ldtpDebug and self.hashmap:
-                #        assert own_name not in self.hashmap, ('Name Conflict in Hashmap. ' + \
-                #                                              own_name + 'already present\n' + foo)
-                #    self.hashmap [own_name] = widget (objectInfo.objType, name, parentName, accessible.getIndexInParent ())
-                if self.hashmap and own_name in self.hashmap:
-                    own_name = self.get_non_conflicting_name (own_name)
+                #        assert ownName not in self.hashmap, ('Name Conflict in Hashmap. ' + \
+                #                                              ownName + 'already present\n' + foo)
+                #    self.hashmap [ownName] = widget (objectInfo.objType, name, parentName, accessible.getIndexInParent ())
+                if self.hashmap and ownName in self.hashmap:
+                    ownName = self.get_non_conflicting_name (ownName)
                 if _ldtpDebug and self.hashmap:
-                    assert own_name not in self.hashmap, ('Name Conflict in Hashmap. ' + \
-                                                          own_name + 'already present\n' + foo)
-                self.hashmap [own_name] = widget (objectInfo.objType, name, parentName, accessible.getIndexInParent ())
-                # if _ldtpDebug and _ldtpDebug == '2':
-                #     print u"%s={class=%s, %s=%s, child_index=%d, parent=%s}" % (own_name, objectInfo.objType,
+                    assert ownName not in self.hashmap, ('Name Conflict in Hashmap. ' + \
+                                                          ownName + 'already present\n' + foo)
+                self.hashmap [ownName] = widget (objectInfo.objType, name, parentName,
+                                                 accessible.getIndexInParent (),
+                                                 ownName, objHierarchyName)
+                if _ldtpDebug and _ldtpDebug == '2':
+                #     print u"%s={class=%s, %s=%s, child_index=%d, parent=%s}" % (ownName, objectInfo.objType,
                 #                                                                 label, name,
                 #                                                                 accessible.getIndexInParent (),
                 #                                                                 parentName)
-                parentName = own_name
+                    print "%s={class=%s, %s=%s, child_index=%d, parent=%s, key=%s, obj_index=%s}" % \
+                        (ownName, objectInfo.objType, label, name, accessible.getIndexInParent (),
+                         parentName, ownName, objHierarchyName)
+                parentName = ownName
+            elif overloadedWindowName is True:
+                ownName = accessible.name
             else:
-                own_name = objectInfo.prefix + str (objectInfo.instIndex)
+                ownName = "%s%d" % (objectInfo.prefix, objectInfo.instIndex)
                 if flag:
-                    if self.applicationmap != {} and own_name in self.applicationmap:
+                    if self.applicationmap != {} and ownName in self.applicationmap:
                         if _ldtpDebug:
-                            print 'has_key', own_name
+                            print 'has_key', ownName
                         # Since the window info is already in appmap, don't process further
                         return
-                    cur_win = window (objectInfo.objType, name, parentName, accessible.getIndexInParent ())
-                    self.applicationmap [own_name] = cur_win
-                    self.hashmap = cur_win.children
+                    curWindow = window (objectInfo.objType, name, parentName,
+                                        accessible.getIndexInParent (),
+                                        ownName, objHierarchyName)
+                    self.applicationmap [ownName] = curWindow
+                    self.hashmap = curWindow.children
                 else:
-                    self.hashmap [own_name] = widget (objectInfo.objType, parentName, accessible.getIndexInParent ())
+                    if self.hashmap and ownName in self.hashmap:
+                        ownName = self.get_non_conflicting_name (ownName)
+                        end_num = re.search ('\d*$', ownName)
+                        if end_num.start () != end_num.end (): # Numbers not postfixed yet
+                            objHierarchyName = "%s#%s" % (ownName [:end_num.start ()],
+                                                          ownName [end_num.start ():])
+                    self.hashmap [ownName] = widget (objectInfo.objType, parentName,
+                                                     accessible.getIndexInParent (),
+                                                     ownName, objHierarchyName)
                 if _ldtpDebug and _ldtpDebug == '2':
-                    print "%s={class=%s, child_index=%d, parent=%s}" % (own_name, objectInfo.objType, \
-                          accessible.getIndexInParent (), parentName)
-                parentName = own_name
+                    print "%s={class=%s, child_index=%d, parent=%s, key=%s, obj_index=%s}" % \
+                        (ownName, objectInfo.objType, accessible.getIndexInParent (),
+                         parentName, ownName, objHierarchyName)
+                parentName = ownName
         for i in range (accessible.childCount):
             child = accessible.getChildAtIndex (i)
             if child is None:
@@ -503,7 +586,7 @@ class appmap:
                 self.objectInst.reset ()
             # time.sleep (1.0 / pow (10, 2))
             try:
-                self.dumpTree (child, parentName)
+                self.doRemap (child, parentName)
             except:
                 if _ldtpDebug:
                     if hasattr (traceback, 'format_exc'):
@@ -512,6 +595,47 @@ class appmap:
                         print traceback.print_exc ()
             child.unref ()
 
+    def push (self, head, value):
+        if self.nodeIndex < 0:
+            self.nodeIndex = 0
+        head [self.nodeIndex] = value
+        self.nodeIndex += 1
+
+    def pop (self, head):
+        value = head.pop (self.nodeIndex, -1)
+        self.nodeIndex -= 1
+        return value
+
+    def tracePath2Parent (self, htContext, context, htComponent, cctxt):
+        if htContext is None or htComponent is None or context is None or cctxt is None:
+            return None
+        head = None
+        self.nodeIndex = 0
+        self.push (head, htContext.index)
+        parent = htContext.parent
+        while 1:
+            tmp = context
+            if re.search (' ', context):
+                tmp = self.escapeChar (context)
+            if context is None or parent is None or \
+                    context == parent or \
+                    re.search (context, parent) or \
+                    re.search (tmp, parent):
+                break
+            component = self.getProperty (htContext, parent)
+            if component:
+                prop = self.getProperty (component, "child_index")
+                if prop is None:
+                    return None
+                self.push (head, prop)
+                parent = self.getProperty (component, "parent")
+                if _ldtpDebug:
+                    print "Parent name: %s - Context name: %s - child index: %d" % \
+                        (parent, context, prop)
+            else:
+                break
+        return head
+
     def isRoleWindowType (self, _role):
         if _role == Accessibility.ROLE_FRAME or _role == Accessibility.ROLE_DIALOG or \
                _role == Accessibility.ROLE_ALERT or _role == Accessibility.ROLE_FONT_CHOOSER or \
@@ -592,7 +716,7 @@ class appmap:
             _windowName = ''
             if self.isRoleWindowType (_role):
                 _windowName = _name = self.getWinNameAppmapFormat (accessible.name, _role)
-                self.hierarchy = '%(window)s#%(objtype)s:%(index)d:%(name)s;%(hierarchy)s' % \
+                self.hierarchy = '%(window)s|%(objtype)s:%(index)d:%(name)s;%(hierarchy)s' % \
                                 {'window' : _windowName, 'objtype' : objectInfo.objType,
                                 'index' : accessible.getIndexInParent (), \
                                 'name' : _name, 'hierarchy' : self.hierarchy}
@@ -666,11 +790,12 @@ class appmap:
             child.unref ()
         return None
 
-    def getChildWindowHandle (self, accessible,  name):
+    def getChildWindowHandle (self, accessible, name):
+        flag = False
         if accessible is None:
-            return None
+            return None, None, flag
         if _ldtpDebug:
-            print 'childCount',  accessible.childCount
+            print 'childCount', accessible.childCount
         for i in range (0, accessible.childCount):
             try:
                 _app = accessible.getChildAtIndex (i)
@@ -684,16 +809,21 @@ class appmap:
             if _app is None:
                 continue
             _role = _app.getRole ()
+            windowName = self.getWinNameAppmapFormat (_app.name, _role)
             if _ldtpDebug:
-                print 'Role: ',  _app.getRoleName (),  name
+                print 'Role: ', _app.getRoleName (), name
                 if _app.name != None:
-                    print '_app.name',  _app.name,  self.getWinNameAppmapFormat (_app.name,  _role)
+                    print '_app.name', _app.name, windowName
             if _app.name is not None and _app.name != '' and \
-                (re.search (name,  _app.name) != None or \
-                re.search (name,  self.getWinNameAppmapFormat (_app.name,  _role)) != None):
-                return _app
+                (re.search (name, _app.name) != None or \
+                re.search (name, windowName) != None):
+                return _app, windowName
             _app.unref ()
-        return None
+        return None, None, flag
+
+    def getAccessibleWindowHandle (self, accessible, context):
+        if accessible:
+            return self.getChildWindowHandle (accessible, context)
 
     def getWindowHandle (self, accessible):
         if accessible is None:
@@ -714,13 +844,128 @@ class appmap:
             _parent.unref ()
         return _handle
 
-    def getContextHandle ():
+    def getWindowList (self):
+        windowList = []
+        _desktop = self.registry.getDesktop (0)
+        if _desktop is None:
+            return windowList
+        for i in range (0, _desktop.childCount):
+            try:
+                _app = _desktop.getChildAtIndex (i)
+            except:
+                if _ldtpDebug:
+                    print traceback.print_exc ()
+                continue
+            if _app is None:
+                continue
+            if _app.name is not None and _app.name != "":
+                windowList.append (_app.name)
+            _app.unref ()
+        return windowList
+
+    def updateWindowAppmapHandle (cctxt):
+        flag   = False
+        child  = None
+        parent = None
+        context    = None
+        windowName = None
+        parentName = None
+        newHashTable = None
+
+        context = cctxt.req.context
+
+        if context is None:
+            return LdtpErrorCode.ARGUMENT_NONE
+        child, windowName = self.getWindowHandle (cctxt.appHandle, context)
+        if child is None:
+            return LdtpErrorCode.WIN_NOT_OPEN
+        cctxt.windowName = windowName
+        parent = child.parent
+        if flag is False and parent:
+            # FIXME
+            return
+
+    def getAccessibleContextHandle (self, appHandle, context):
+        if appHandle is None or context is None:
+            return None
+        contextPattern = re.compile (context)
+        if self.lastExistingContext:
+            self.lastExistingContextPattern = re.compile (self.lastExistingContext)
+        childCount = appHandle.childCount
+        if _ldtpDebug and childCount and appHandle.name:
+            print 'getAccessibleContextHandle: %s %ld' % (appHandle.name, childCount)
+        tmpTable = []
+        windowIndex = 0
+        windowTmpIndex = 1
+        for i in range (appHandle.childCount):
+            child = appHandle.getChildAtIndex (i)
+            if child is None:
+                continue
+            childRole = child.getRole ()
+            childName = child.name
+            windowName = None
+            if childName:
+                windowName = self.getWinNameAppmapFormat (childName, childRole)
+            if windowName is None or re.search ("0", windowName):
+                windowName = self.getWinNameAppmapFormat ("%d" % windowIndex, childRole)
+                windowIndex += 1
+            if windowName:
+                if windowName in tmpTable is False:
+                    tmpTable.append (windowName)
+                else:
+                    if childName:
+                        windowName = self.getWinNameAppmapFormat ("%s%d" % (childName, windowTmpIndex),
+                                                                  childRole)
+                        windowTmpIndex += 1
+                    if windowName and windowName in tmpTable is False:
+                        tmpTable.append (windowName)
+            if self.lastNewContext and self.lastExistingContext:
+                if contextPattern and childName and self.lastExistingContextPattern and \
+                        (re.search (self.lastExistingContextPattern, context) or \
+                        (windowName and re.search (contextPattern, windowName))):
+                    # FIXME: Localization related implementation
+                    tmp = escapeChar (childName, ' ')
+                    if self.lastNewContext is childName or \
+                            self.lastNewContext is tmp or \
+                            re.search (self.lastNewContext, childName):
+                        return child
+            # FIXME: Localization related implementation
+            if _ldtpDebug and childName:
+                print 'Context: %s Child name: %s' % (context, childName)
+            if _ldtpDebug and windowName:
+                print 'Window name: %s' % windowName
+            if contextPattern and childName and \
+                    (re.search (contextPattern, childName) or \
+                         (windowName and re.search (contextPattern, windowName))):
+                if _ldtpDebug:
+                    print "Matched - Context: %s Child name: %s" % (context, childName)
+                return child
+            else:
+                windowName = None
+                child.unref ()
+
+    def getAccessibleComponentHandle (self):
         return None
 
-    def getComponentHandle ():
+    def getComponentHandle (self, cctxt, windowProp, context, component):
+        head      = None
+        classType = None
+        curWindow = None
+        timeWait  = self.objTimeout
+        curComponent    = None
+        contextHandle   = None
+        componentHandle = None
+        currTime = prevTime = time.time ()
+
+        if self.objTimeout is 0:
+            timeWait = 5
+
+        while currTime - prevTime < timeWait:
+            self.updateWindowAppmapHandle (cctxt)
+
         return None
 
-    def getAppHandle (self,  appName):
+    def getAppHandle (self, appName):
         # App handle should be freed, if not None
         if appName == None or appName == '':
             return None
@@ -743,20 +988,20 @@ class appmap:
             _name = ''
             if _app.name is not None and _app.name != "":
                 if _ldtpDebug:
-                    print 'DEBUG _app.name, _app.getRoleName',  _app.name,  _app.getRoleName ()
-                _name = self.getWinNameAppmapFormat (_app.name,  _app.getRole ())
+                    print 'DEBUG _app.name, _app.getRoleName', _app.name, _app.getRoleName ()
+                _name = self.getWinNameAppmapFormat (_app.name, _app.getRole ())
                 if _ldtpDebug:
-                    print '_name',  _name
+                    print '_name', _name
             else:
                 _app.unref ()
                 continue
             if _ldtpDebug:
-                print 'appName',  appName,  _name,  _app.name
-            if _name != None and re.search (appName,  _name) != None or \
-                re.search (appName,  _app.name) != None:
+                print 'appName', appName, _name, _app.name
+            if _name != None and re.search (appName, _name) != None or \
+                re.search (appName, _app.name) != None:
                 return _app
             else:
-                _appHandle = self.getChildWindowHandle (_app,  appName)
+                _appHandle, windowName, flag = self.getChildWindowHandle (_app, appName)
                 if _appHandle != None:
                     _app.unref ()
                     return _appHandle
@@ -792,6 +1037,25 @@ class appmap:
             return re.match (searchObjInfo.pattern, key)
         return False
 
+    def searchBasedOnIndex (self, value, searchObjInfo):
+        if value is None or searchObjInfo is None or searchObjInfo.key is None:
+            return False
+        flag = False
+        if value.type is not None:
+            if value.type == 'frame' or value.type == 'dialog' or \
+               value.type == 'alert' or value.type == 'file_chooser' or \
+               value.type == 'font_chooser':
+                flag = True
+        if value.label is None or value.label == '':
+            return False
+        if (searchObjInfo.window is False or flag is True) and \
+               (value.hierarchy == searchObjInfo.key or \
+                (searchObjInfo.pattern is not None and \
+                 searchObjInfo.pattern != '' and \
+                 re.search (searchObjInfo.pattern, value.hierarchy) is not None)):
+            return True
+        return False
+
     def searchBasedOnLabel (self, value, searchObjInfo):
         if value is None or searchObjInfo is None or searchObjInfo.key is None:
             return False
@@ -845,6 +1109,12 @@ class appmap:
         if context in ht.children:
             return ht.children [context]
         obj = SearchObjInfo (context, isWindow, context)
+        if re.search ('#', context):
+            # Search based on index
+            # ex: btn#1, mnu#1, etc
+            for widget in ht.children.keys ():
+                if self.searchBasedOnIndex (ht.children [widget], obj):
+                    return widget
         for widget in ht.children.keys ():
             if self.searchBasedOnKey (widget, obj):
                 return widget
@@ -855,3 +1125,16 @@ class appmap:
             if self.searchAfterStrippingSpace ():
                 return widget
         return None
+
+    def getProperty (self, ht, property):
+        if ht is None or ht.children is None:
+            if _ldtpDebug:
+                print getErrorMessage (LdtpErrorCode.APPMAP_NOT_INITIALIZED)
+            return None
+        if property is None:
+            if _ldtpDebug:
+                print getErrorMessage (LdtpErrorCode.ARGUMENT_NONE)
+            return None
+        if property in ht.children:
+            return ht.children [property]
+        return None
