@@ -265,7 +265,7 @@ class state:
 
 class LdtpErrorCode:
     SUCCESS = 0
-    ARGUMENT_NULL = -1001
+    ARGUMENT_NONE = -1001
     ACCEPT_FAILED = -1002
     UNABLE_TO_REINIT_LDTP = -1003
     UNABLE_TO_ALLOCATE_MEMORY = -1004
@@ -447,6 +447,7 @@ class LdtpErrorCode:
     CHECK_BOX_STATE_NOT_ENABLED = -1180
     ERROR_PUSH_BUTTON_STATE_NOT_ENABLED = -1181
     ERROR_TOGGLE_BUTTON_STATE_NOT_ENABLED = -1182
+    UNABLE_TO_GET_WINDOW_LIST = -1183
 
 class XmlTags:
     XML_HEADER                   = "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
@@ -491,7 +492,7 @@ class ConnectionLost (Exception):
     def __str__ (self):
         return repr (self.value)
 
-def startLdtpExecutionEngine ():
+def connect2LdtpExecutionEngine ():
     _display = os.getenv ('DISPLAY')
     
     if _display == None:
@@ -584,6 +585,10 @@ def waitnanoseconds (nanoSeconds = 30000.0):
     except TypeError:
         time.sleep (30000.0 / pow (10, 9))
 
+def timeElapsed (self, prevTime):
+    currTime = int (time.time () - prevTime)
+    return currTime
+
 # Send given packet to server
 def sendpacket (msg):
     flag = False
@@ -603,12 +608,15 @@ def sendpacket (msg):
             # Get client socket fd based on thread id
             client = sockFdPool.get (threading.currentThread ())
             if client is None:
-                _mainSock, _ldtpUseTcp = startLdtpExecutionEngine ()
+                _mainSock, _ldtpUseTcp = connect2LdtpExecutionEngine ()
                 client = sockFdPool [threading.currentThread ()] = _mainSock
 
             # Encode the message in UTF-8 so we don't break on extended
             # characters in the application GUIs
-            buf = msg.encode ('utf8')
+            try:
+                buf = msg.encode ('utf-8')
+            except UnicodeDecodeError:
+                buf = unicode (msg, 'utf-8').encode ('utf-8')
 			
             # Pack length (integer value) in network byte order
             msglen = struct.pack ('!i', len (buf))
@@ -620,7 +628,7 @@ def sendpacket (msg):
                 print 'Send packet', buf
             #_sendLck.release ()
         except socket.error, msg:
-            _mainSock, _ldtpUseTcp = startLdtpExecutionEngine ()
+            _mainSock, _ldtpUseTcp = connect2LdtpExecutionEngine ()
             sockFdPool [threading.currentThread ()] = _mainSock
             raise LdtpExecutionError ('Server aborted')
         except:
@@ -706,8 +714,8 @@ def getErrorMessage (errorCode, args):
     msg = None
     if errorCode == LdtpErrorCode.SUCCESS: 
         msg = "Successfully completed"
-    elif errorCode == LdtpErrorCode.ARGUMENT_NULL:
-        msg = "Argument cannot be NULL. Please check the arguments."
+    elif errorCode == LdtpErrorCode.ARGUMENT_NONE:
+        msg = "Argument cannot be None. Please check the arguments."
     elif errorCode == LdtpErrorCode.ACCEPT_FAILED:
         msg = "Error occurred while accepting request from client.  Please retry."
     elif errorCode == LdtpErrorCode.UNABLE_TO_REINIT_LDTP:
