@@ -27,7 +27,7 @@
 
 __author__ = "Nagappan A"
 __maintainer__ = "Nagappan A"
-__version__ = "1.2.0"
+__version__ = "1.3.0"
 
 import re
 import os
@@ -153,18 +153,18 @@ class record:
         try:
             return self.getObjectNameDebug (name)
         except:
-            if hasattr (traceback, 'format_exc'):
-                print traceback.format_exc ()
-            else:
-                print traceback.print_exc ()
+            if _ldtpDebug:
+                if hasattr (traceback, 'format_exc'):
+                    print traceback.format_exc ()
+                else:
+                    print traceback.print_exc ()
             return ''
 
     def getObjectNameDebug (self, name):
-        if _ldtpDebug:
-            print name
-        _windowName = re.split ('#', name, 1)
-        if _ldtpDebug:
-            print _windowName,  len (_windowName)
+        _windowName = re.split ('\|', name, 1)
+        if _ldtpDebug and _windowName:
+            print 'Name: %s' % name
+            print "_windowName", _windowName, len (_windowName)
         _name = ''
         if len (_windowName) == 2:
             _name = _windowName [1]
@@ -211,7 +211,7 @@ class record:
                         parent.unref ()
                     try:
                         self.objectInst.reset ()
-                        self.appmap.dumpTree (_accessible,  name,  True)
+                        self.appmap.doRemap (_accessible,  name,  True)
                     except:
                         if _ldtpDebug:
                             if hasattr (traceback, 'format_exc'):
@@ -336,7 +336,7 @@ class record:
                 parent.unref ()
             try:
                 self.objectInst.reset ()
-                self.appmap.dumpTree (event.source, name)
+                self.appmap.doRemap (event.source, name)
                 if self.__pckt and (event.type == 'window:create' or event.type == 'window:destroy'):
                     _xml = self.__pckt.generatenotificationxml (self.__windowName, \
                                                               '', \
@@ -364,7 +364,7 @@ class record:
                 _parent.unref ()
             try:
                 self.objectInst.reset ()
-                self.appmap.dumpTree (event.source, _name)
+                self.appmap.doRemap (event.source, _name)
                 if self.__pckt and (event.type == 'window:create' or event.type == 'window:destroy'):
                     _xml = self.__pckt.generatenotificationxml (self.__windowName, \
                                                               '', \
@@ -637,8 +637,8 @@ class record:
                     _parent.unref ()
                 self.__tableCell = _name
                 self.__tableName = self.appmap.hierarchy
-                if re.search ('#', self.appmap.hierarchy) == None:
-                    self.appmap.hierarchy = _windowName + '#' + self.appmap.hierarchy
+                if re.search ('|', self.appmap.hierarchy) == None:
+                    self.appmap.hierarchy = _windowName + '|' + self.appmap.hierarchy
                 if _ldtpDebug:
                     print 'selectrow (\'' + _windowName + '\', \'' + self.appmap.hierarchy + \
                           '\', \'' + _name + '\')' + str (event.type)
