@@ -42,6 +42,14 @@ from xml.dom.minidom import *
 from xml.parsers.expat import ExpatError
 from xml.sax import saxutils
 
+#Wildcard Specific Globals
+appList = []
+APPHOME = "%s/.ldtp" % os.environ['HOME']
+APPROOT = "/usr/share/ldtp"
+
+#ooldtp support Global
+contextList = []
+
 # Let us not register our application under at-spi application list
 os.environ ['GTK_MODULES'] = ''
 
@@ -636,6 +644,77 @@ def addWait (callbackFunc, timeElapsed):
     except ValueError:
         pass
 
+#Wildcard specific function... Get applist
+def getapps ():
+    appList = []
+    path = "%s/appdata" % APPHOME
+    if os.path.exists (path):
+	appListPath = path
+    else:
+	print "Home copy not available"
+	path = "%s/appdata" % APPROOT
+	if os.path.exists (path):
+            appListPath = path
+        else:
+            print "Even root copy not available"
+	    return appList
+    f = open (appListPath)
+    for item in f:
+        appList.append (item.strip ())
+    f.close ()
+    return appList
+
+#Wildcard specific function... Replace with wildcards
+def wildcard (str):
+    global appList
+    if len (appList) == 0:
+        appList = getapps ()
+    for item in appList:
+        regex= '\"\S*?%s\S*?\"' % item
+        item = '\"*%s*\"' % item
+        str = re.compile (regex).sub (item, str, 1)
+    return str
+
+#Wildcard specific function... Save applist
+def saveAppList (newAppList):
+    global appList
+    appList = newAppList
+    path = "%s/appdata" % APPHOME
+    if os.path.exists (APPHOME) == False:
+        os.mkdir (APPHOME, 0755)
+    f = open (path, 'w')
+    for item in appList:
+        item = "%s\n" % item
+        f.write (item)
+    f.close ()
+
+#oldtp support specific function... initContext
+def initContext():
+    global contextList
+    contextList = []
+
+#ooldtp support specific function... objectOrient
+def objectOrient (line):
+    global contextList
+    re1 = re.compile ('\".*?\"')
+    shrink = re1.findall (line)[0]
+    re3 = re.compile ('\W|^\d*')
+    context = re3.sub ('', shrink)
+    re2 = re.compile ('\".*?\"\,\s')
+    if re2.search (line) != None:
+        parts = re2.split (line, 1)
+    else:
+        parts = re1.split (line, 1)
+    for all in contextList:
+        if all == context:
+            line = '%s.%s%s' % (context, parts[0], parts[1])
+            return line
+    contextList.append (context)
+    #last = len(context) - 1
+    line = '%s = context(%s)\n\t' % (context, shrink)
+    line = '%s%s.%s%s' % (line, context, parts[0], parts[1])
+    return line
+
 def callback (guiCallbackFunc, data, timeElapsed = None):
     global generatedCode
     if timeElapsed is not None:
@@ -654,6 +733,8 @@ def callback (guiCallbackFunc, data, timeElapsed = None):
                 tmpData = "\\n"
             data = "%s%s%s" % (data, tmpLine, tmpData)
         data += "\n"
+    #get Windownames replaced by wildcards here
+    data = wildcard (data)
     generatedCode += data
     guiCallbackFunc (data)
 
@@ -849,7 +930,7 @@ def processDataDebug (pckt, callbackFunc):
                     continue
                 elif _pcktInfo.objectType == 'text':
                     _objName = _pcktInfo.objectName
-                    if re.search ('#', _objName) is not None:
+                    if re.search ('|', _objName) is not None:
                         _responseData = getObjectName (_pcktInfo.objectName)
                         if _responseData is not None:
                             _objName = _responseData
@@ -877,7 +958,7 @@ def processDataDebug (pckt, callbackFunc):
                     continue
                 elif _pcktInfo.objectType == 'right click':
                     _objName = _pcktInfo.objectName
-                    if re.search ('#', _objName) is not None:
+                    if re.search ('|', _objName) is not None:
                         _responseData = getObjectName (_pcktInfo.objectName)
                         if _responseData is not None:
                             _objName = _responseData
@@ -891,7 +972,7 @@ def processDataDebug (pckt, callbackFunc):
                 elif _pcktInfo.objectType == 'panel' and \
                      _pcktInfo.eventType.startswith ('object:state-changed:focused'):
                     _objName = _pcktInfo.objectName
-                    if re.search ('#', _objName) is not None:
+                    if re.search ('|', _objName) is not None:
                         _responseData = getObjectName (_pcktInfo.objectName)
                         if _responseData is not None:
                             _objName = _responseData
