@@ -313,21 +313,17 @@ class component:
 
     def gettextvalue (self, *args):
         length = len (args)
-        if length < 1:
-            raise ldtp.LdtpExecutionError ('Min 1 argument required')
         name = self.__name
-        textProperty  = ''
         startPosition = None
-        endPosition    = None
+        endPosition   = None
 
         if length >= 2:
-            if isinstance (args [1], str):
-                textProperty = args [1]
+            if isinstance (args [0], str):
                 name = args [0]
+                if length >= 2:
+                    startPosition = args [1]
                 if length >= 3:
-                    startPosition = args [2]
-                if length >= 4:
-                    endPosition = args [3]
+                    endPosition = args [2]
             else:
                 name = self.getName ('')
                 textProperty = args [0]
@@ -339,7 +335,7 @@ class component:
         if name == '':
             raise ldtp.LdtpExecutionError ('Component name is required')
 
-        return ldtp.gettextvalue (self.__window, name, textProperty, startPosition, endPosition)
+        return ldtp.gettextvalue (self.__window, name, startPosition, endPosition)
 
     def grabfocus (self, name = ''):
         return ldtp.grabfocus (self.__window, self.getName (name))
