@@ -169,8 +169,9 @@ generate_response_packet (LDTPClientContext* cctxt, LDTPErrorCode* err, char **r
 	if (cctxt->resp->data && cctxt->resp->data_len > 0) {
 		char *external_tmp_xml_file = getenv ("LDTP_EXTERNAL_TEMP_FILE");
 		if (!ldtp_script_service &&
-		    (ldtp_external_xml_file || (external_tmp_xml_file != NULL &&
-						g_ascii_strcasecmp (external_tmp_xml_file, "2") == 0)) &&
+		    (ldtp_external_xml_file ||
+		     (external_tmp_xml_file != NULL &&
+		      g_ascii_strcasecmp (external_tmp_xml_file, "1") == 0)) &&
 		    cctxt->resp->data_len > 512) {
 			char *filename = write2file (cctxt->resp->data, cctxt->resp->data_len);
 			if (filename) {
@@ -697,13 +698,19 @@ remove_app_list_table (gpointer key, gpointer value, gpointer user_data)
 static void
 update_context_name (LDTPClientContext* cctxt)
 {
-	char *window_name = NULL;
-	gboolean flag = FALSE;
 	if (!cctxt)
 		return;
 	if (cctxt->req && cctxt->req->context) {
-		Accessible *context_handle = NULL;
-		context_handle = get_window_handle (NULL, cctxt->req->context, &window_name, &flag);
+		char *window_name = NULL;
+		gboolean flag = FALSE;
+		Accessible *context_handle = get_window_handle (NULL, cctxt->req->context, &window_name, &flag);
+		if (window_name) {
+			g_free (cctxt->req->context);
+			cctxt->req->context = g_strdup (window_name);
+			g_free (window_name);
+		}
+		Accessible_unref (context_handle);
+		/*
 		if (context_handle) {
 			char *name = Accessible_getName (context_handle);
 
@@ -716,9 +723,10 @@ update_context_name (LDTPClientContext* cctxt)
 			}
 			Accessible_unref (context_handle);
 		}
+		if (window_name)
+			g_free (window_name);
+		*/
 	}
-	if (window_name)
-		g_free (window_name);
 }
 
 static void
@@ -990,8 +998,7 @@ handle_request (LDTPClientContext* cctxt,
 	case LDTP_CMD_REMAP:
 		if (cctxt->req->component == NULL) {
 			update_cur_window_appmap_handle (cctxt, err);
-		}
-		else {
+		} else {
 			update_cur_context_appmap_handle (cctxt, err);
 		}
 		return;
@@ -1008,7 +1015,7 @@ handle_request (LDTPClientContext* cctxt,
 	case LDTP_CMD_GETAPPLIST: {
 		char *data = NULL;
 		GHashTable *table;
-		table = get_window_list ();
+		table = get_app_list ();
 		if (table) {
 			g_hash_table_foreach (table, &add_item_to_list, (gpointer)&data);
 			cctxt->resp->data = data;
@@ -1023,22 +1030,19 @@ handle_request (LDTPClientContext* cctxt,
 					cctxt->resp->data = tmp;
 					g_print ("LIST: %s\n", cctxt->resp->data);
 					cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
-				}
-				else {
+				} else {
 					*err = LDTP_ERROR_UNABLE_TO_GET_APPLICATION_LIST;
 					g_print ("%s\n", ldtp_error_get_message (*err));
 					log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
 				}
-			}
-			else {
+			} else {
 				*err = LDTP_ERROR_UNABLE_TO_GET_APPLICATION_LIST;
 				g_print ("%s\n", ldtp_error_get_message (*err));
 				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
 			}
 			//g_hash_table_foreach_remove (table, remove_app_list_table, NULL);
 			g_hash_table_destroy (table);
-		}
-		else {
+		} else {
 			*err = LDTP_ERROR_UNABLE_TO_GET_APPLICATION_LIST;
 			g_print ("%s - %s\n", __FILE__, ldtp_error_get_message (*err));
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
@@ -1046,16 +1050,11 @@ handle_request (LDTPClientContext* cctxt,
 		return;
 	}
 	case LDTP_CMD_GETWINDOWLIST: {
-		if (cctxt->app_map) {
-			char *data = NULL;
-			/*
-			  If appliction map is not initialized for new windows,
-			  then call to update_window_list will get those windows too
-			  NOTE: update_window_list will fail if none of the applications
-			  has been accessed, ie., atleast one component function has
-			  to be called in one of the current running application 
-			*/
-			g_hash_table_foreach (cctxt->app_map, &add_item_to_list, (gpointer)&data);
+		char *data = NULL;
+		GHashTable *table;
+		table = get_window_name_list ();
+		if (table) {
+			g_hash_table_foreach (table, &add_item_to_list, (gpointer)&data);
 			cctxt->resp->data = data;
 			if (cctxt->resp->data) {
 				char *tmp = NULL;
@@ -1068,21 +1067,19 @@ handle_request (LDTPClientContext* cctxt,
 					cctxt->resp->data = tmp;
 					g_print ("LIST: %s\n", cctxt->resp->data);
 					cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
-				}
-				else {
-					*err = LDTP_ERROR_UNABLE_TO_GET_OBJECT_LIST;
+				} else {
+					*err = LDTP_ERROR_UNABLE_TO_GET_WINDOW_LIST;
 					g_print ("%s\n", ldtp_error_get_message (*err));
 					log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
 				}
-			}
-			else {
-				*err = LDTP_ERROR_UNABLE_TO_GET_OBJECT_LIST;
+			} else {
+				*err = LDTP_ERROR_UNABLE_TO_GET_WINDOW_LIST;
 				g_print ("%s\n", ldtp_error_get_message (*err));
 				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
 			}
-		}
-		else {
-			*err = LDTP_ERROR_APPMAP_NOT_INITIALIZED;
+			g_hash_table_destroy (table);
+		} else {
+			*err = LDTP_ERROR_UNABLE_TO_GET_WINDOW_LIST;
 			g_print ("%s - %s\n", __FILE__, ldtp_error_get_message (*err));
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
 		}
@@ -1136,16 +1133,18 @@ handle_request (LDTPClientContext* cctxt,
 			if (tmp) {
 				g_free (data);
 				cctxt->resp->data = tmp;
-				g_print ("LIST: %s\n", cctxt->resp->data);
-				cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
-			}
-			else {
+				// FIXME: Verified in us-en environment, need to verify in localization env
+				cctxt->resp->data_len = strlen (cctxt->resp->data);
+				if (ldtp_debug)
+					g_print ("LIST: %ld - %s\n",
+						 cctxt->resp->data_len,
+						 cctxt->resp->data);
+			} else {
 				*err = LDTP_ERROR_UNABLE_TO_GET_OBJECT_LIST;
 				g_print ("%s\n", ldtp_error_get_message (*err));
 				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
 			}
-		}
-		else {
+		} else {
 			*err = LDTP_ERROR_UNABLE_TO_GET_OBJECT_LIST;
 			g_print ("%s\n", ldtp_error_get_message (*err));
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
@@ -1218,7 +1217,7 @@ handle_request (LDTPClientContext* cctxt,
 				cctxt->resp->data = tmp;
 				if (ldtp_debug)
 					g_print ("LIST: %s\n", tmp);
-				cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
+				cctxt->resp->data_len = strlen (cctxt->resp->data);
 			} else {
 				g_free (cctxt->resp->data);
 				*err = LDTP_ERROR_UNABLE_TO_GET_CHILD_WITH_PROVIDED_ROLE;
@@ -1302,14 +1301,12 @@ handle_request (LDTPClientContext* cctxt,
 				cctxt->resp->data = tmp;
 				g_print ("LIST: %s\n", cctxt->resp->data);
 				cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
-			}
-			else {
+			} else {
 				*err = LDTP_ERROR_UNABLE_TO_GET_OBJECT_LIST;
 				g_print ("%s\n", ldtp_error_get_message (*err));
 				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
 			}
-		}
-		else {
+		} else {
 			*err = LDTP_ERROR_UNABLE_TO_GET_OBJECT_LIST;
 			g_print ("%s\n", ldtp_error_get_message (*err));
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
