@@ -185,6 +185,7 @@ get_key_value (gchar *keyval)
 		/* This will identify small characters */
 		if (*keyval >= 'a' && *keyval <= 'z') {
 			return_val.shift = FALSE;
+			return_val.capslck = FALSE;
 			return_val.non_print_key = FALSE;
 			return_val.value = Char_Key_Synth_Vals[*keyval-'a'];
 			return return_val;
@@ -193,7 +194,8 @@ get_key_value (gchar *keyval)
 		/* This will identify Capital Charaters i.e. Shift+Small
 		   Character */
 		else if (*keyval >= 'A' && *keyval <= 'Z') {
-			return_val.shift = TRUE;
+			return_val.shift = FALSE;
+			return_val.capslck = TRUE;
 			return_val.non_print_key = FALSE;
 			return_val.value = Char_Key_Synth_Vals[*keyval-'A'];
 			return return_val;
@@ -202,6 +204,7 @@ get_key_value (gchar *keyval)
 		/* This will identify Digits */
 		else if (*keyval >= '0' && *keyval <='9') {
 			return_val.shift = FALSE;
+			return_val.capslck = FALSE;
 			return_val.non_print_key = FALSE;
 			return_val.value = Digit_Key_Synth_Vals[*keyval-'0'];
 			return return_val;
@@ -213,6 +216,7 @@ get_key_value (gchar *keyval)
 			for (index = 0; index < DOWNCHAR; index++) 
 				if (*keyval == Symbol_Key_Synth_Vals[index].sym) {
 					return_val.shift = FALSE;
+					return_val.capslck = FALSE;
 					return_val.non_print_key = FALSE;
 					return_val.value = Symbol_Key_Synth_Vals[index].KeyVal;
 					return return_val;
@@ -222,6 +226,7 @@ get_key_value (gchar *keyval)
 			for ( ; index < DOWNCHAR + UPCHAR; index++)
 				if (*keyval == Symbol_Key_Synth_Vals[index].sym) {
 					return_val.shift = TRUE;
+					return_val.capslck = FALSE;
 					return_val.non_print_key = FALSE;
 					return_val.value = Symbol_Key_Synth_Vals[index].KeyVal;
 					return return_val;
@@ -234,6 +239,7 @@ get_key_value (gchar *keyval)
 		for (index = 0; NonPrint_Key_Synth_Vals[index].sym; index++)
 			if (g_ascii_strcasecmp (NonPrint_Key_Synth_Vals[index].sym, keyval) == 0) {
 				return_val.shift = FALSE;
+				return_val.capslck = FALSE;
 				return_val.non_print_key = TRUE;
 				return_val.value = NonPrint_Key_Synth_Vals[index].KeyVal;
 				return return_val;
@@ -242,6 +248,7 @@ get_key_value (gchar *keyval)
      
 	/* Key Undefined */
 	return_val.shift = FALSE;
+	return_val.capslck = FALSE;
 	return_val.non_print_key = FALSE;
 	return_val.value = UNDEFINED_KEY;
 	return return_val;
@@ -317,9 +324,17 @@ generate_keyboard_sequence (char *input_string, FILE *log_fp)
 			type = SPI_KEY_PRESSRELEASE;
 		if (keyval[index].shift == 1)
 			SPI_generateKeyboardEvent (50, NULL, SPI_KEY_PRESS); // press shift
+		if (keyval[index].capslck == 1) {
+			SPI_generateKeyboardEvent (66, NULL, SPI_KEY_PRESS); // press capslck
+			SPI_generateKeyboardEvent (66, NULL, SPI_KEY_RELEASE); // release capslck
+		}
 		flag = SPI_generateKeyboardEvent (keyval[index].value, NULL, type);
 		if (keyval[index].shift == 1)
 			SPI_generateKeyboardEvent (50, NULL, SPI_KEY_RELEASE); // release shift
+		if (keyval[index].capslck == 1) {
+			SPI_generateKeyboardEvent (66, NULL, SPI_KEY_PRESS); // press capslck
+			SPI_generateKeyboardEvent (66, NULL, SPI_KEY_RELEASE); // release capslck
+		}
 		if (flag == FALSE) {
 			error = LDTP_ERROR_UNABLE_TO_ENTER_KEY;
 			goto error;
