@@ -120,9 +120,9 @@ select_labels_panel_by_name (Accessible *object, FILE *log_fp)
 	if (parent)
 		Accessible_unref (parent);
 	if (flag)
-		error =  (LDTP_ERROR_SUCCESS);
+		error = LDTP_ERROR_SUCCESS;
 	else
-		error =  (LDTP_ERROR_UNABLE_TO_SELECT_LABEL);
+		error = LDTP_ERROR_UNABLE_TO_SELECT_LABEL;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 	return error;
 }
@@ -132,18 +132,17 @@ label_main (LDTPClientContext* cctxt, int command)
 {
 	LDTPErrorCode error;
 	switch (command) {
-	case LDTP_CMD_SELECTLABELSPANELBYNAME: {
+	case LDTP_CMD_SELECTLABELSPANELBYNAME:
 		error = select_labels_panel_by_name (cctxt->gui_handle->handle, cctxt->log_fp);
 		break;
-	}
 	case LDTP_CMD_GETLABEL:
 		cctxt->resp->data = get_label (cctxt->gui_handle->handle);
 		if (cctxt->resp->data) {
 			cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
-			error =  (LDTP_ERROR_SUCCESS);
+			error = LDTP_ERROR_SUCCESS;
 		}
 		else
-			error =  (LDTP_ERROR_LABEL_NOT_FOUND);
+			error = LDTP_ERROR_LABEL_NOT_FOUND;
 		break;
 	case LDTP_CMD_GETLABELATINDEX: {
 		long index = 0;
@@ -152,15 +151,14 @@ label_main (LDTPClientContext* cctxt, int command)
 		cctxt->resp->data = get_label_at_index (cctxt->gui_handle->handle, index);
 		if (cctxt->resp->data) {
 			cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
-			error =  (LDTP_ERROR_SUCCESS);
+			error = LDTP_ERROR_SUCCESS;
 		}
 		else
-			error =  (LDTP_ERROR_LABEL_NOT_FOUND);
+			error = LDTP_ERROR_LABEL_NOT_FOUND;
 	}
 		break;
 	default:
-		error = ( (LDTP_ERROR_COMMAND_NOT_IMPLEMENTED));
-		break;
+		error = LDTP_ERROR_COMMAND_NOT_IMPLEMENTED;
 	}
 	return error;
 }
