@@ -8,6 +8,7 @@
  *    Premkumar J <jpremkumar@novell.com>
  *
  * Copyright 2004 - 2006 Novell, Inc.
+ * Copyright 2007 - 2008 Nagappan Alagappan
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -285,12 +286,12 @@ static gboolean
 search_key_based (gpointer key, gpointer value, gpointer user_data)
 {
 	OBJInfo *obj = (OBJInfo *) user_data;
-	if (!key || !user_data || !obj->key) {
+	if (!key || !obj || !obj->key) {
 		return FALSE;
 	}
 	if (g_pattern_match_simple (obj->key, key)) {
 		if (ldtp_debug && obj->key && key) {
-			g_print ("glob- key: %s %s\n",
+			g_print ("Key: %s %s\n",
 				 (gchar *) key,
 				 obj->key);
 		}
@@ -298,7 +299,38 @@ search_key_based (gpointer key, gpointer value, gpointer user_data)
 			gchar *parent = get_property (value, "parent", NULL);
 			if (parent && g_pattern_match_simple (obj->parent_name, parent))
 				return TRUE;
-		}
+		} else
+			return TRUE;
+	}
+	return FALSE;
+}
+
+static void
+search_window_key_based (gpointer key, gpointer value, gpointer user_data)
+{
+	if (search_key_based (key, value, user_data)) {
+		OBJInfo *obj;
+		obj = (OBJInfo *) user_data;
+		g_hash_table_insert (obj->ht_data_list, g_strdup ((gchar *) key), value);
+	}
+}
+
+static gboolean
+search_obj_based_on_index (gpointer key, gpointer value, gpointer user_data)
+{
+	gchar *obj_index = NULL;
+	OBJInfo *obj = (OBJInfo *) user_data;
+	if (!value || !obj || !obj->key) {
+		return FALSE;
+	}
+	obj_index = get_property (value, "obj_index", NULL);
+	if (obj_index && g_pattern_match_simple (obj->key, obj_index)) {
+		if (obj->parent_name) {
+			gchar *parent = get_property (value, "parent", NULL);
+			if (parent && g_pattern_match_simple (obj->parent_name, parent))
+				return TRUE;
+		} else
+			return TRUE;
 	}
 	return FALSE;
 }
@@ -317,7 +349,7 @@ search_key_glob_based (gpointer key, gpointer value, gpointer user_data)
 	*/
 	obj = (OBJInfo *) user_data;
 	if (ldtp_debug && obj->key && key) {
-		g_print ("glob- key: %d - %d - %d - %s %s %ld %ld\n",
+		g_print ("glob - key: %d - %d - %d - %s %s %ld %ld\n",
 			 obj->obj_is_window,
 			 g_pattern_match_simple ("frm*", obj->key),
 			 g_pattern_match_simple ("dlg*", obj->key),
@@ -325,14 +357,6 @@ search_key_glob_based (gpointer key, gpointer value, gpointer user_data)
 			 g_utf8_strlen (obj->key, -1));
 	}
 	if (obj->obj_is_window) {
-		if (obj->key && g_utf8_collate (key, obj->key) == 0) {
-			if (obj->parent_name && value) {
-				gchar *parent = get_property (value, "parent", NULL);
-				if (parent && g_pattern_match_simple (obj->parent_name, parent))
-					return TRUE;
-			} else
-				return TRUE;
-		}
 		if (obj->pattern) {
 			if (g_pattern_match_string (obj->pattern, key)) {
 				if (obj->parent_name && value) {
@@ -434,6 +458,8 @@ search_label_based (gpointer key, gpointer value, gpointer user_data)
 	if (!value || !user_data) {
 		return FALSE;
 	}
+	if (key)
+		g_print ("Key: %s\n", (gchar *) key);
 	obj = (OBJInfo *) user_data;
 	g_hash_table_lookup_extended (value, "class", NULL, (gpointer) &class);
 	g_hash_table_lookup_extended (value, "label", NULL, (gpointer) &label);
@@ -455,7 +481,7 @@ search_label_based (gpointer key, gpointer value, gpointer user_data)
 	  if type and key matches then return true
 	*/
 	if (label && tmp_data) {
-		if ((gchar *)key && label && tmp_data)
+		if (ldtp_debug && (gchar *)key && label && tmp_data)
 			g_print ("Label: %s - %s - %s\n", (gchar *)key, (gchar *)tmp_data, label);
 		if (!obj->obj_is_window || flag)
 			if (g_utf8_collate (label, tmp_data) == 0) {
@@ -635,23 +661,24 @@ static gboolean
 search_obj_after_stripping_space (gpointer key, gpointer value, gpointer user_data)
 {
 	gchar *tmp;
-	if (!key && !user_data)
+	OBJInfo *obj = (OBJInfo *) user_data;
+	if (!key || !user_data || !obj->key)
 		return FALSE;
-	if (g_utf8_strchr (user_data, -1, ' ') == NULL)
+	if (g_utf8_strchr (obj->key, -1, ' ') == NULL)
 		return FALSE;
-	tmp = escape_character (user_data, ' ');
+	tmp = escape_character (obj->key, ' ');
 	if (!tmp)
 		return FALSE;
-	g_print ("DEBUG: %s - %s - %s\n", tmp, (gchar *)user_data, (gchar *)key);
-	if (g_utf8_collate (key, tmp) == 0 || g_utf8_collate (key, user_data) == 0) {
+	g_print ("DEBUG: %s - %s - %s\n", tmp, (gchar *)obj->key, (gchar *)key);
+	if (g_utf8_collate (key, tmp) == 0 || g_utf8_collate (key, obj->key) == 0) {
 		g_free (tmp);
 		return TRUE;
 	}
 	else {
 		gchar *glob_str = NULL;
 		/*
-		  FIXME: Use get_object_info function to get the the type of object
-		  and based on that use the convention, instead of directly using *
+		    FIXME: Use get_object_info function to get the the type of object
+		      and based on that use the convention, instead of directly using *
 		*/
 		glob_str = g_strdup_printf ("*%s", tmp);
 		g_print ("Glob str: %s - %s - %s\n", glob_str, (gchar *)key, tmp);
@@ -675,7 +702,7 @@ search_window_after_stripping_space (gpointer key, gpointer value, gpointer user
 	if (!key || !user_data || !value)
 		return;
 	obj = (OBJInfo *) user_data;
-	if (search_obj_after_stripping_space (key, value, obj->key)) {
+	if (search_obj_after_stripping_space (key, value, obj)) {
 		if (obj && obj->ht_data_list && key && value)
 			g_hash_table_insert (obj->ht_data_list, g_strdup ((gchar *) key), value);
 	}
@@ -690,33 +717,40 @@ get_all_matching_def (GHashTable *ht, gchar *context, gboolean obj_is_window, FI
 	OBJInfo obj;
 	GPatternSpec *pattern = NULL;
 	if (ht == NULL) {
-		err =  (LDTP_ERROR_APPMAP_NOT_INITIALIZED);
+		err = LDTP_ERROR_APPMAP_NOT_INITIALIZED;
 		g_print ("%s - %d - %s\n", __FILE__, __LINE__, ldtp_error_get_message (err));
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (err), log_fp);
 		return NULL;
 	}
 	if (context == NULL) {
-		err =  (LDTP_ERROR_ARGUMENT_NULL);
+		err = LDTP_ERROR_ARGUMENT_NULL;
 		g_print ("%s\n", ldtp_error_get_message (err));
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (err), log_fp);
 		return NULL;
 	}
-	/*
-	  Search key based on glob expression
-	*/
-	g_print ("Search key based on glob expression: %s\n", context);
-	if (context)
-		pattern = g_pattern_spec_new ((const gchar *)context);
-
 	ht_data_list = g_hash_table_new_full (&g_str_hash, &g_str_equal,
 						 destroy_string, NULL);
-	obj.obj_is_window = obj_is_window;
-	obj.pattern = pattern;
 	obj.key = context;
-	obj.ht_data_list = ht_data_list;
-	g_hash_table_foreach (ht, search_window_key_glob_based, &obj);
-	if (pattern)
-		g_pattern_spec_free (pattern);
+	obj.parent_name = NULL;
+	obj.obj_is_window = obj_is_window;
+	g_hash_table_foreach (ht, search_window_key_based, &obj);
+
+	if (!g_hash_table_size (ht_data_list)) {
+		/*
+		  Search key based on glob expression
+		*/
+		g_print ("Search key based on glob expression: %s %s %d\n", context, __FILE__, __LINE__);
+		if (context)
+			pattern = g_pattern_spec_new ((const gchar *)context);
+
+		obj.obj_is_window = obj_is_window;
+		obj.pattern = pattern;
+		obj.key = context;
+		obj.ht_data_list = ht_data_list;
+		g_hash_table_foreach (ht, search_window_key_glob_based, &obj);
+		if (pattern)
+			g_pattern_spec_free (pattern);
+	}
 	if (!g_hash_table_size (ht_data_list)) {
 		/*
 		  Search label based
@@ -724,6 +758,9 @@ get_all_matching_def (GHashTable *ht, gchar *context, gboolean obj_is_window, FI
 		g_print ("Search label based: %s\n", context);
 
 		obj.pattern = NULL;
+		obj.key = context;
+		obj.parent_name = NULL;
+		obj.obj_is_window = obj_is_window;
 		g_hash_table_foreach (ht, search_window_label_based, &obj);
 	}
 	if (!g_hash_table_size (ht_data_list)) {
@@ -738,6 +775,9 @@ get_all_matching_def (GHashTable *ht, gchar *context, gboolean obj_is_window, FI
 			pattern = g_pattern_spec_new (context);
 
 		obj.pattern = pattern;
+		obj.key = context;
+		obj.parent_name = NULL;
+		obj.obj_is_window = obj_is_window;
 		g_hash_table_foreach (ht, search_window_label_glob_based, &obj);
 		if (pattern)
 			g_pattern_spec_free (pattern);
@@ -747,6 +787,9 @@ get_all_matching_def (GHashTable *ht, gchar *context, gboolean obj_is_window, FI
 		  Search key after stripping space
 		*/
 		g_print ("Search key after stripping space: %s\n", context);
+		obj.key = context;
+		obj.parent_name = NULL;
+		obj.obj_is_window = obj_is_window;
 		g_hash_table_foreach (ht, search_window_after_stripping_space, &obj);
 	}
 	return ht_data_list;
@@ -761,7 +804,7 @@ match_object_def (gchar *key, gchar *value, gboolean obj_is_window)
 	/*
 	  Search key based on glob expression
 	*/
-	g_print ("Search key based on glob expression: %s\n", key);
+	g_print ("Search key based on glob expression: %s %s %d\n", key, __FILE__, __LINE__);
 	if (key)
 		pattern = g_pattern_spec_new (/*(const ggchar *)*/key);
 
@@ -816,20 +859,20 @@ match_object_def (gchar *key, gchar *value, gboolean obj_is_window)
 GHashTable*
 get_object_def (GHashTable *ht, gchar *context, gchar *parent, FILE *log_fp, gboolean obj_is_window)
 {
-	GHashTable *ht_context = NULL;
-	LDTPErrorCode err;
-	gchar msg [256];
 	OBJInfo obj;
-	GPatternSpec *pattern = NULL;
+	gchar msg [256];
+	LDTPErrorCode err;
+	GPatternSpec *pattern  = NULL;
+	GHashTable *ht_context = NULL;
 
 	if (ht == NULL) {
-		err =  (LDTP_ERROR_APPMAP_NOT_INITIALIZED);
+		err = LDTP_ERROR_APPMAP_NOT_INITIALIZED;
 		g_print ("%s - %d - %s\n", __FILE__, __LINE__, ldtp_error_get_message (err));
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (err), log_fp);
 		return NULL;
 	}
 	if (context == NULL) {
-		err =  (LDTP_ERROR_ARGUMENT_NULL);
+		err = LDTP_ERROR_ARGUMENT_NULL;
 		g_print ("%s\n", ldtp_error_get_message (err));
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (err), log_fp);
 		return NULL;
@@ -837,24 +880,36 @@ get_object_def (GHashTable *ht, gchar *context, gchar *parent, FILE *log_fp, gbo
 	/*
 	  Search given context in hash table
 	*/
-	//g_hash_table_lookup_extended (ht, (gconstpointer) context, NULL, (gpointer) ht_context);
-	obj.key = context;
-	obj.parent_name = parent;
-	ht_context = g_hash_table_find (ht, search_key_based, &obj);
-	if (ht_context)
-		return ht_context;
+	if (g_utf8_strchr (context, -1, '#')) {
+		/*
+		  Search key based on object index
+		  ex: btn#1, mnu#1
+		*/
+		g_print ("Search based on index: %s\n", context);
+		obj.key = context;
+		obj.parent_name = parent;
+		obj.obj_is_window = obj_is_window;
+		ht_context = g_hash_table_find (ht, search_obj_based_on_index, &obj);
+	}
+	if (!ht_context) {
+		g_print ("Search based on key: %s\n", context);
+		obj.key = context;
+		obj.parent_name = parent;
+		obj.obj_is_window = obj_is_window;
+		ht_context = g_hash_table_find (ht, search_key_based, &obj);
+	}
 	if (!ht_context) {
 		/*
 		  Search key based on glob expression
 		*/
-		g_print ("Search key based on glob expression: %s - %d\n", context, obj_is_window);
+		g_print ("Search key based on glob expression: %s - %d %s %d\n", context, obj_is_window, __FILE__, __LINE__);
 		if (context)
 			pattern = g_pattern_spec_new ((const gchar *)context);
 
-		obj.obj_is_window = obj_is_window;
 		obj.pattern = pattern;
-		obj.parent_name = parent;
 		obj.key = context;
+		obj.parent_name = parent;
+		obj.obj_is_window = obj_is_window;
 		ht_context = g_hash_table_find (ht, search_key_glob_based, &obj);
 		if (pattern)
 			g_pattern_spec_free (pattern);
@@ -867,10 +922,10 @@ get_object_def (GHashTable *ht, gchar *context, gchar *parent, FILE *log_fp, gbo
 
 		g_print ("Search label based: %s - %d\n", context, obj_is_window);
 
-		obj.obj_is_window = obj_is_window;
 		obj.pattern = NULL;
-		obj.parent_name = parent;
 		obj.key = context;
+		obj.parent_name = parent;
+		obj.obj_is_window = obj_is_window;
 		ht_context = g_hash_table_find (ht, search_label_based, &obj);
 	}
 	if (!ht_context) {
@@ -884,10 +939,10 @@ get_object_def (GHashTable *ht, gchar *context, gchar *parent, FILE *log_fp, gbo
 		if (context)
 			pattern = g_pattern_spec_new (context);
 
-		obj.obj_is_window = obj_is_window;
 		obj.pattern = pattern;
-		obj.parent_name = parent;
 		obj.key = context;
+		obj.parent_name = parent;
+		obj.obj_is_window = obj_is_window;
 		ht_context = g_hash_table_find (ht, search_label_glob_based, &obj);
 		if (pattern)
 			g_pattern_spec_free (pattern);
@@ -897,8 +952,10 @@ get_object_def (GHashTable *ht, gchar *context, gchar *parent, FILE *log_fp, gbo
 		  Search key after stripping space
 		*/
 		g_print ("Search key after stripping space: %s\n", context);
+		obj.key = context;
 		obj.parent_name = parent;
-		ht_context = g_hash_table_find (ht, search_obj_after_stripping_space, context);
+		obj.obj_is_window = obj_is_window;
+		ht_context = g_hash_table_find (ht, search_obj_after_stripping_space, &obj);
 	}
 	if (!ht_context) {
 		g_sprintf (msg, "Object definition %s not found in appmap", context);
@@ -932,13 +989,13 @@ get_property (GHashTable *ht, gchar *property, FILE *log_fp)
 	LDTPErrorCode err;
 
 	if (ht == NULL) {
-		err =  (LDTP_ERROR_APPMAP_NOT_INITIALIZED);
+		err = LDTP_ERROR_APPMAP_NOT_INITIALIZED;
 		g_print ("%s - %d - %s\n", __FILE__, __LINE__, ldtp_error_get_message (err));
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (err), log_fp);
 		return NULL;
 	}
 	if (property == NULL) {
-		err =  (LDTP_ERROR_ARGUMENT_NULL);
+		err = LDTP_ERROR_ARGUMENT_NULL;
 		g_print ("%s\n", ldtp_error_get_message (err));
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (err), log_fp);
 		return NULL;
