@@ -30,42 +30,46 @@
 struct obj_property {
 	gboolean appmap_type; /* TRUE, remap generated appamp. FALSE, initappmap based appmap*/
 	long child_index;
-	char *str_child_index;
-	char *parent_name;
+	gchar *str_child_index;
+	gchar *parent_name;
 	int class_type; /* SPI_ROLE_MENU_ITEM */
-	char *class_name; /* menu_item */
-	char *label;
-	char *label_by;
-	char *app_name;
+	gchar *class_name; /* menu_item */
+	gchar *label;
+	gchar *label_by;
+	gchar *app_name;
 };
 
 struct unkn_label_property { /* Example: txt0, pnl2*/
 	gboolean appmap_type; /* TRUE, remap generated appamp. FALSE, initappmap based appmap*/
 	long child_index;
-	char *str_child_index;
-	char *parent_name;
-	char *obj_name;
+	gchar *str_child_index;
+	gchar *parent_name;
+	gchar *obj_name;
 };
 
 struct obj_info {
-	char *key;
+	gchar *key;
+	gchar *parent_name;
 	gboolean obj_is_window;
 	GPatternSpec *pattern;
+	GHashTable *ht_data_list;
 };
 
 typedef struct obj_info OBJInfo;
 typedef struct obj_property OBJProperty;
 typedef struct unkn_label_property UnknLabelProperty;
 
-GHashTable *appmap_init (char *, FILE *);
+GHashTable *appmap_init (gchar *, FILE *);
 void   ldtp_appmap_free (GHashTable* appmap);
-char   *get_property (GHashTable *, char *, FILE *);
-GHashTable *get_object_def (GHashTable *, char *, FILE *, gboolean);
-void print_attributes (char *key, char *value, char *userdata);
-void print_component (char *key, GHashTable *component, char *userdata);
-void print_context (char *key, GHashTable *context, char *userdata);
-void key_destroy_func (gpointer data);
-void value_destroy_func (gpointer data);
+gchar   *get_property (GHashTable *, gchar *, FILE *);
+GHashTable *get_object_def (GHashTable *, gchar *, gchar *, FILE *, gboolean);
+GHashTable *get_all_matching_def (GHashTable *ht, gchar *context, gboolean obj_is_window, FILE *log_fp);
+void print_attributes (gchar *key, gchar *value, gchar *userdata);
+void print_component (gchar *key, GHashTable *component, gchar *userdata);
+void print_context (gchar *key, GHashTable *context, gchar *userdata);
+void destroy_string (gpointer data);
+void destroy_hashtable (gpointer data);
+gboolean match_object_def (gchar *key, gchar *value, gboolean obj_is_window);
 gboolean remove_context_entries (gpointer key, gpointer value, gpointer context);
 gboolean remove_remapped_entry (gpointer key, gpointer value, gpointer user_data);
 gboolean search_title_based (gpointer key, gpointer value, gpointer user_data);
