@@ -53,6 +53,9 @@ ldtp_error_get_message (LDTPErrorCode error)
 	case LDTP_ERROR_UNABLE_TO_GET_APPLICATION_LIST:
 		msg = "Unable to get application list";
 		break;
+	case LDTP_ERROR_UNABLE_TO_GET_WINDOW_LIST:
+		msg = "Unable to get window list";
+		break;
 	case LDTP_ERROR_UNABLE_TO_GET_OBJECT_LIST:
 		msg = "Unable to get object list";
 		break;
