@@ -208,7 +208,8 @@ typedef enum _LDTPErrorCode {
 	LDTP_ERROR_UNABLE_TO_GET_CHILD_WITH_PROVIDED_ROLE,
 	LDTP_ERROR_UNABLE_TO_GET_PAGE_TAB_NAME,
 	LDTP_ERROR_UNABLE_TO_VERIFY_PAGE_TAB_NAME,
-	LDTP_ERROR_CHECK_BOX_STATE_NOT_ENABLED
+	LDTP_ERROR_CHECK_BOX_STATE_NOT_ENABLED,
+	LDTP_ERROR_UNABLE_TO_GET_WINDOW_LIST
 } LDTPErrorCode;
 
 const char* ldtp_error_get_message (LDTPErrorCode error);
