@@ -8,6 +8,7 @@
  *    Poornima <pnayak@novell.com>
  *
  * Copyright 2004 - 2006 Novell, Inc.
+ * Copyright 2007 - 2008 Nagappan Alagappan
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -226,8 +227,7 @@ trace_path_to_parent (GHashTable *context, char *context_name,
 	if (!head) {
 		log_msg (LDTP_LOG_CAUSE, "Unable to initialize stack", log_fp);
 		return NULL;
-	}
-	else {
+	} else {
 		char *tmp = NULL;
 		prop = get_property (component, "child_index", log_fp);
 		if (!prop) {
@@ -245,12 +245,11 @@ trace_path_to_parent (GHashTable *context, char *context_name,
 				char *str = escape_character (context_name, ' ');
 				tmp = g_strdup_printf ("*%s", str);
 				g_free (str);
-			}
-			else
+			} else
 				tmp = g_strdup_printf ("*%s", context_name);
 			g_print ("TMP: %s\n", tmp);
 			if (!context_name || !parent_name ||
-			    !g_utf8_collate (context_name, parent_name) ||
+			    g_utf8_collate (context_name, parent_name) == 0 ||
 			    g_pattern_match_simple (context_name, parent_name) ||
 			    g_pattern_match_simple (tmp, parent_name)) {
 				g_free (tmp);
@@ -478,10 +477,10 @@ get_accessible_app_handle (char *app_name)
 }
 
 /*
-  Get window list
+  Get application name list
 */
 GHashTable*
-get_window_list (void)
+get_app_list (void)
 {
 	long i, num_apps;
 	Accessible *desktop;
@@ -540,6 +539,107 @@ get_window_list (void)
 	return table;
 }
 
+/*
+  Get window name list
+*/
+GHashTable*
+get_window_name_list (void)
+{
+	long i, j = 0, k = 1;
+	long num_apps;
+	long app_index;
+	long child_count;
+	Accessible *child;
+	Accessible *desktop;
+	Accessible *app_handle;
+	GHashTable *table = NULL;
+	AccessibleRole role;
+	char *tmp  = NULL;
+	char *name = NULL;
+	char *window_name = NULL;
+
+	/*
+	  Get SPI handle of desktop 0
+	  FIXME: Need to change the hardcoded value of desktop
+	*/
+	desktop = SPI_getDesktop (0);
+	if (desktop == NULL)
+		return NULL;
+
+	table = g_hash_table_new_full (&g_str_hash, &g_str_equal, destroy_string, NULL);
+	/*
+	  Get the total count of childs in the desktop
+	*/
+	num_apps = Accessible_getChildCount (desktop);
+	g_print ("Number of application: %ld\n", num_apps);
+	/*
+	  Traverse the list of application handle and if application
+	  name matches, then return the handle
+	*/
+	for (app_index = 0; app_index < num_apps; app_index++) {
+		app_handle = Accessible_getChildAtIndex (desktop, app_index);
+		if (!app_handle)
+			continue;
+		child_count = Accessible_getChildCount (app_handle);
+		for (i = 0; i < child_count; i++) {
+			child = Accessible_getChildAtIndex (app_handle, i);
+			if (!child)
+				continue;
+			/*
+			  Application name of child handle
+			*/
+			name = Accessible_getName (child);
+			role = Accessible_getRole (child);
+			if (name) {
+				g_print ("child_name: %s\n", name);
+				window_name = get_window_name_in_appmap_format (name, role);
+			}
+			if (!window_name) {
+				gchar *tmp = g_strdup_printf ("%ld", j++);
+				if (tmp) {
+					window_name = get_window_name_in_appmap_format (tmp, role);
+					g_free (tmp);
+					tmp = NULL;
+				}
+			}
+			if (window_name && table) {
+				g_print ("window_name: %s\n", window_name);
+				if (g_hash_table_lookup_extended (table, window_name, NULL, NULL) == FALSE) {
+					g_hash_table_insert (table, g_strdup (window_name), NULL);
+					/* Enable for debugging purpose
+					   g_hash_table_foreach (tmpTable, (GHFunc)&print_tmp_table, NULL);
+					*/
+				} else {
+					if (name)
+						tmp = g_strdup_printf ("%s%ld", name, k++);
+					if (window_name) {
+						g_free (window_name);
+						window_name = NULL;
+					}
+					if (tmp) {
+						window_name = get_window_name_in_appmap_format (tmp, role);
+						g_free (tmp);
+						tmp = NULL;
+					}
+					if (window_name && g_hash_table_lookup_extended (table,
+											 window_name,
+											 NULL, NULL) == FALSE)
+						g_hash_table_insert (table, g_strdup (window_name), NULL);
+				}
+			}
+			SPI_freeString (name);
+			Accessible_unref (child);
+			if (window_name) {
+				g_free (window_name);
+				window_name = NULL;
+			}
+		}
+		Accessible_unref (app_handle);
+	}
+	Accessible_unref (desktop);
+	return table;
+}
+
 /*
  * To get accessible handle of the context (window)
  */
@@ -566,141 +666,141 @@ get_accessible_context_handle (Accessible *app_handle, char *context)
 		return NULL;
 	}
 
-	context_pattern = g_pattern_spec_new (context);
-	if (last_existing_context)
-		last_existing_context_pattern = g_pattern_spec_new (last_existing_context);
-
 	child_count = Accessible_getChildCount (app_handle);
 	name = Accessible_getName (app_handle);
 	if (ldtp_debug && name)
 		g_print ("Child count: %ld of %s\n", child_count, name);
 	SPI_freeString (name);
-	if (child_count > 0) {
-		tmpTable = g_hash_table_new_full (&g_str_hash, &g_str_equal, destroy_string, NULL);
-		for (i = 0; i < child_count; i++) {
-			child = Accessible_getChildAtIndex (app_handle, i);
-			if (!child)
-				continue;
-			/*
-			  Application name of child handle
-			*/
-			child_name = Accessible_getName (child);
-			role = Accessible_getRole (child);
-			if (child_name) {
-				g_print ("child_name: %s\n", child_name);
-				window_name = get_window_name_in_appmap_format (child_name, role);
+	if (child_count  < 1)
+		return;
+	context_pattern = g_pattern_spec_new (context);
+	if (last_existing_context)
+		last_existing_context_pattern = g_pattern_spec_new (last_existing_context);
+
+	tmpTable = g_hash_table_new_full (&g_str_hash, &g_str_equal, destroy_string, NULL);
+	for (i = 0; i < child_count; i++) {
+		child = Accessible_getChildAtIndex (app_handle, i);
+		if (!child)
+			continue;
+		/*
+		  Application name of child handle
+		*/
+		child_name = Accessible_getName (child);
+		role = Accessible_getRole (child);
+		if (child_name) {
+			g_print ("child_name: %s\n", child_name);
+			window_name = get_window_name_in_appmap_format (child_name, role);
+		}
+		if (!window_name) {
+			gchar *tmp = g_strdup_printf ("%ld", j++);
+			if (tmp) {
+				window_name = get_window_name_in_appmap_format (tmp, role);
+				g_free (tmp);
+				tmp = NULL;
 			}
-			if (!window_name) {
-				gchar *tmp = g_strdup_printf ("%ld", j++);
+		}
+		if (window_name && tmpTable) {
+			g_print ("window_name: %s\n", window_name);
+			if (g_hash_table_lookup_extended (tmpTable, window_name, NULL, NULL) == FALSE) {
+				g_hash_table_insert (tmpTable, g_strdup (window_name), NULL);
+				/* Enable for debugging purpose
+				   g_hash_table_foreach (tmpTable, (GHFunc)&print_tmp_table, NULL);
+				*/
+			} else {
+				if (child_name)
+					tmp = g_strdup_printf ("%s%ld", child_name, k++);
+				if (window_name) {
+					g_free (window_name);
+					window_name = NULL;
+				}
 				if (tmp) {
 					window_name = get_window_name_in_appmap_format (tmp, role);
 					g_free (tmp);
 					tmp = NULL;
 				}
-			}
-			if (window_name && tmpTable) {
-				g_print ("window_name: %s\n", window_name);
-				if (g_hash_table_lookup_extended (tmpTable, window_name, NULL, NULL) == FALSE) {
+				if (window_name && g_hash_table_lookup_extended (tmpTable,
+										 window_name,
+										 NULL, NULL) == FALSE) {
 					g_hash_table_insert (tmpTable, g_strdup (window_name), NULL);
-					/* Enable for debugging purpose
-					   g_hash_table_foreach (tmpTable, (GHFunc)&print_tmp_table, NULL);
-					*/
-				} else {
-					if (child_name)
-						tmp = g_strdup_printf ("%s%ld", child_name, k++);
-					if (window_name) {
-						g_free (window_name);
-						window_name = NULL;
-					}
-					if (tmp) {
-						window_name = get_window_name_in_appmap_format (tmp, role);
-						g_free (tmp);
-						tmp = NULL;
-					}
-					if (window_name && g_hash_table_lookup_extended (tmpTable,
-											  window_name,
-											  NULL, NULL) == FALSE) {
-						g_hash_table_insert (tmpTable, g_strdup (window_name), NULL);
-					}
 				}
 			}
-			if (last_new_context && last_existing_context) {
-				g_print ("Last new: **%s** - existing: %s - context: %s - **%s**\n",
-					 last_new_context, last_existing_context, context, child_name);
-				if (context_pattern && child_name && last_existing_context_pattern &&
-				    (g_pattern_match_string (last_existing_context_pattern, context) ||
-				     (window_name && g_pattern_match_string (context_pattern, window_name)))) {
-#ifdef ENABLE_LOCALIZATION
-					if ((temp = ldtp_compare_with_locale (last_new_context, child_name))) {
-						g_free (temp);
-						g_free (window_name);
-						SPI_freeString (child_name);
-						if (last_existing_context_pattern)
-							g_pattern_spec_free (last_existing_context_pattern);
-						if (context_pattern)
-							g_pattern_spec_free (context_pattern);
-						g_hash_table_destroy (tmpTable);
-						return child;
-					}
-#else
-					char *tmp = escape_character (child_name, ' ');
-					if (g_utf8_collate (last_new_context, child_name) == 0 ||
-					    g_utf8_collate (last_new_context, tmp) == 0 ||
-					    g_pattern_match_simple (last_new_context, child_name)) {
-						g_free (tmp);
-						g_free (window_name);
-						SPI_freeString (child_name);
-						if (last_existing_context_pattern)
-							g_pattern_spec_free (last_existing_context_pattern);
-						if (context_pattern)
-							g_pattern_spec_free (context_pattern);
-						g_hash_table_destroy (tmpTable);
-						return child;
-					}
-					g_free (tmp);
-#endif
-				}
-			}
-#ifdef ENABLE_LOCALIZATION
-			if (context && child_name &&
-			    (temp = ldtp_compare_with_locale (context, child_name))) {
-				g_print ("Context: %s In_Locale: %s Child_name: %s\n", context, temp, child_name);
-				g_free (temp);
-				g_free (window_name);
-				SPI_freeString (child_name);
-				if (last_existing_context_pattern)
-					g_pattern_spec_free (last_existing_context_pattern);
-				if (context_pattern)
-					g_pattern_spec_free (context_pattern);
-				g_hash_table_destroy (tmpTable);
-				return child;
-			}
-#else
-			if (ldtp_debug && context && child_name)
-				g_print ("Context: %s Child_name: %s\n", context, child_name);
-			if (ldtp_debug && window_name)
-				g_print ("Window name: %s\n", window_name);
-			if (context_pattern && child_name &&
-			    (g_pattern_match_string (context_pattern, child_name) ||
+		}
+		if (last_new_context && last_existing_context) {
+			g_print ("Last new: **%s** - existing: %s - context: %s - **%s**\n",
+				 last_new_context, last_existing_context, context, child_name);
+			if (context_pattern && child_name && last_existing_context_pattern &&
+			    (g_pattern_match_string (last_existing_context_pattern, context) ||
 			     (window_name && g_pattern_match_string (context_pattern, window_name)))) {
-				g_free (window_name);
-				g_print ("Matched - Context: %s Child_name: %s\n", context, child_name);
-				SPI_freeString (child_name);
-				if (last_existing_context_pattern)
-					g_pattern_spec_free (last_existing_context_pattern);
-				if (context_pattern)
-					g_pattern_spec_free (context_pattern);
-				g_hash_table_destroy (tmpTable);
-				return child;
-			}
+#ifdef ENABLE_LOCALIZATION
+				if ((temp = ldtp_compare_with_locale (last_new_context, child_name))) {
+					g_free (temp);
+					g_free (window_name);
+					SPI_freeString (child_name);
+					if (last_existing_context_pattern)
+						g_pattern_spec_free (last_existing_context_pattern);
+					if (context_pattern)
+						g_pattern_spec_free (context_pattern);
+					g_hash_table_destroy (tmpTable);
+					return child;
+				}
+#else
+				char *tmp = escape_character (child_name, ' ');
+				if (g_utf8_collate (last_new_context, child_name) == 0 ||
+				    g_utf8_collate (last_new_context, tmp) == 0 ||
+				    g_pattern_match_simple (last_new_context, child_name)) {
+					g_free (tmp);
+					g_free (window_name);
+					SPI_freeString (child_name);
+					if (last_existing_context_pattern)
+						g_pattern_spec_free (last_existing_context_pattern);
+					if (context_pattern)
+						g_pattern_spec_free (context_pattern);
+					g_hash_table_destroy (tmpTable);
+					return child;
+				}
+				g_free (tmp);
 #endif
-			else {
-				g_free (window_name);
-				window_name = NULL;
-				SPI_freeString (child_name);
-				Accessible_unref (child);
 			}
 		}
+#ifdef ENABLE_LOCALIZATION
+		if (context && child_name &&
+		    (temp = ldtp_compare_with_locale (context, child_name))) {
+			g_print ("Context: %s In_Locale: %s Child_name: %s\n", context, temp, child_name);
+			g_free (temp);
+			g_free (window_name);
+			SPI_freeString (child_name);
+			if (last_existing_context_pattern)
+				g_pattern_spec_free (last_existing_context_pattern);
+			if (context_pattern)
+				g_pattern_spec_free (context_pattern);
+			g_hash_table_destroy (tmpTable);
+			return child;
+		}
+#else
+		if (ldtp_debug && context && child_name)
+			g_print ("Context: %s Child_name: %s\n", context, child_name);
+		if (ldtp_debug && window_name)
+			g_print ("Window name: %s\n", window_name);
+		if (context_pattern && child_name &&
+		    (g_pattern_match_string (context_pattern, child_name) ||
+		     (window_name && g_pattern_match_string (context_pattern, window_name)))) {
+			g_free (window_name);
+			g_print ("Matched - Context: %s Child_name: %s\n", context, child_name);
+			SPI_freeString (child_name);
+			if (last_existing_context_pattern)
+				g_pattern_spec_free (last_existing_context_pattern);
+			if (context_pattern)
+				g_pattern_spec_free (context_pattern);
+			g_hash_table_destroy (tmpTable);
+			return child;
+		}
+#endif
+		else {
+			g_free (window_name);
+			window_name = NULL;
+			SPI_freeString (child_name);
+			Accessible_unref (child);
+		}
 	}
 	if (last_existing_context_pattern)
 		g_pattern_spec_free (last_existing_context_pattern);
@@ -869,8 +969,7 @@ is_object_matching (Accessible *object, GHashTable *comp_attributes, FILE *log_f
 				g_print ("Before: %s - After: %s\n", hash_label, str);
 				value = g_utf8_collate (str, accessible_label);
 				g_free (str);
-			}
-			else
+			} else
 				value = g_utf8_collate (hash_label, accessible_label);
 		if (value == 0)
 #endif
@@ -881,13 +980,11 @@ is_object_matching (Accessible *object, GHashTable *comp_attributes, FILE *log_f
 				g_free (value);
 #endif
 				return TRUE;
-			}
-		else {
+			} else {
 			SPI_freeString (accessible_label);
 			return FALSE;
 		}
-	}
-	else {
+	} else {
 		hash_label = get_property (comp_attributes, "label_by", log_fp);
 		if (hash_label) {
 			long len = -1;
@@ -902,8 +999,7 @@ is_object_matching (Accessible *object, GHashTable *comp_attributes, FILE *log_f
 					g_print ("Before: %s - After: %s\n", hash_label, str);
 					value = g_utf8_collate (str, accessible_label);
 					g_free (str);
-				}
-				else
+				} else
 					value = g_utf8_collate (hash_label, accessible_label);
 			if (value == 0)
 #endif
@@ -914,8 +1010,7 @@ is_object_matching (Accessible *object, GHashTable *comp_attributes, FILE *log_f
 					g_free (value);
 #endif
 					return TRUE;
-				}
-			else {
+				} else {
 				g_free (accessible_label);
 				return FALSE;
 			}
@@ -1049,10 +1144,13 @@ get_child_window_handle (Accessible *app_handle, char *context, char **window_na
 				g_pattern_spec_free (pattern);
 			if (tmp_pattern)
 				g_pattern_spec_free (tmp_pattern);
-			//g_hash_table_foreach_remove (tmpTable, (GHRFunc)&remove_tmp_entries, NULL);
 			g_hash_table_destroy (tmpTable);
 			return child;
 		}
+		if (*window_name) {
+			g_free (*window_name);
+			*window_name = NULL;
+		}
 		SPI_freeString (name);
 		Accessible_unref (child);
 	}
@@ -1060,7 +1158,6 @@ get_child_window_handle (Accessible *app_handle, char *context, char **window_na
 		g_pattern_spec_free (pattern);
 	if (tmp_pattern)
 		g_pattern_spec_free (tmp_pattern);
-	//g_hash_table_foreach_remove (tmpTable, (GHRFunc)&remove_tmp_entries, NULL);
 	g_hash_table_destroy (tmpTable);
 	return NULL;
 }
@@ -1204,7 +1301,7 @@ update_cur_window_appmap_handle (LDTPClientContext* cctxt,
 		SPI_freeString (name);
 		Accessible_unref (child);
 		if (cctxt->window_name) {
-			g_print ("cctxt->window_name: %s\n", cctxt->window_name);
+			g_print ("cctxt->window_name: %s %s %d\n", cctxt->window_name, __FILE__, __LINE__);
 			pthread_mutex_lock (&mutex);
 			g_hash_table_replace (cctxt->app_map, g_strdup (cctxt->window_name), new_hashtable);
 			pthread_mutex_unlock (&mutex);
@@ -1215,15 +1312,13 @@ update_cur_window_appmap_handle (LDTPClientContext* cctxt,
 				g_free (window_name);
 				window_name = NULL;
 			}
-		}
-		else {
+		} else {
 			*err = LDTP_ERROR_UNABLE_TO_UPDATE_APPMAP;
 			g_print ("DEBUG: Window name NULL\n");
 			g_hash_table_destroy (new_hashtable);
 			return;
 		}
-	}
-	else {
+	} else {
 		Accessible_unref (child);
 		g_print ("DEBUG: New hash table NULL\n");
 		*err = LDTP_ERROR_UNABLE_TO_UPDATE_APPMAP;
@@ -1251,7 +1346,7 @@ update_cur_context_appmap_handle (LDTPClientContext *cctxt,
 		return;
 	}
 	if (cctxt->app_map == NULL) {
-		*err =  (LDTP_ERROR_APPMAP_NOT_INITIALIZED);
+		*err = LDTP_ERROR_APPMAP_NOT_INITIALIZED;
 		g_print ("%s - %d - %s\n", __FILE__, __LINE__, ldtp_error_get_message (*err));
 		return;
 	}
@@ -1293,7 +1388,7 @@ update_cur_context_appmap_handle (LDTPClientContext *cctxt,
 		SPI_freeString (name);
 		Accessible_unref (gui_handle->handle);
 		if (cctxt->window_name) {
-			g_print ("cctxt->window_name: %s\n", cctxt->window_name);
+			g_print ("cctxt->window_name: %s %s %d\n", cctxt->window_name, __FILE__, __LINE__);
 			pthread_mutex_lock (&mutex);
 			g_hash_table_replace (cctxt->app_map, g_strdup (cctxt->window_name), new_hashtable);
 			pthread_mutex_unlock (&mutex);
@@ -1529,7 +1624,6 @@ ldtp_gui_get_gui_handle (LDTPClientContext* cctxt,
 	GHashTable *cur_component;
 
 	if (!cctxt || !cctxt->req->context) {
-		/* FIXME: Error handling */
 		*err = LDTP_ERROR_ARGUMENT_NULL;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
 		return NULL;
