@@ -50,4 +50,6 @@ void ldtp_request_init (LDTPRequest* req);
 void ldtp_request_fill_request (LDTPRequest* req, 
 				gchar* packet, size_t len,
 				LDTPErrorCode* err);
+void
+ldtp_request_free (LDTPRequest* req, int finalize);
 #endif
