@@ -41,6 +41,10 @@
 #define DATA_ELEMENT                 "<DATA>"
 #define ATTRIBUTE_LENGTH_ELEMENT     "<LENGTH>"
 #define ATTRIBUTE_LENGTH_END_ELEMENT "</LENGTH>"
+#define ATTRIBUTE_FILE_ELEMENT     "<FILE>"
+#define ATTRIBUTE_FILE_END_ELEMENT "</FILE>"
+#define ATTRIBUTE_NAME_ELEMENT     "<NAME>"
+#define ATTRIBUTE_NAME_END_ELEMENT "</NAME>"
 #define ATTRIBUTE_VALUE_ELEMENT      "<VALUE><![CDATA["
 #define ATTRIBUTE_VALUE_END_ELEMENT  "]]></VALUE>"
 #define DATA_END_ELEMENT             "</DATA>"
@@ -61,6 +65,10 @@
 #define DATA_ELEMENT_LEN                  6
 #define ATTRIBUTE_LENGTH_ELEMENT_LEN      8
 #define ATTRIBUTE_LENGTH_END_ELEMENT_LEN  9
+#define ATTRIBUTE_FILE_ELEMENT_LEN        6
+#define ATTRIBUTE_FILE_END_ELEMENT_LEN    7
+#define ATTRIBUTE_NAME_ELEMENT_LEN        6
+#define ATTRIBUTE_NAME_END_ELEMENT_LEN    7
 #define ATTRIBUTE_VALUE_ELEMENT_LEN      16
 #define ATTRIBUTE_VALUE_END_ELEMENT_LEN  11
 #define DATA_END_ELEMENT_LEN              7
