@@ -415,10 +415,10 @@ ldtp_server_thread (void *ptr)
 static void
 ldtp_print (const char *string) 
 {
-	if (ldtp_debug || (env_ldtp_debug != NULL && g_ascii_strcasecmp (env_ldtp_debug, "2") == 0)) {
+	if (ldtp_debug || env_ldtp_debug) {
 		printf ("%s", string);
 	}
-	if (ldtp_debug_log_fp || (env_ldtp_debug_file != NULL && g_ascii_strcasecmp (env_ldtp_debug_file, "2") == 0)) {
+	if (ldtp_debug_log_fp || env_ldtp_debug_file) {
 		fprintf (ldtp_debug_log_fp, "%s", string);
 		fflush (ldtp_debug_log_fp);
 	}
