@@ -112,6 +112,9 @@ push_button_main (LDTPClientContext* cctxt, int command)
         case LDTP_CMD_KBDENTER:
                 error = device_main (cctxt, command);
                 break;
+        case LDTP_CMD_GRABFOCUS:
+                error = grab_focus (cctxt->gui_handle->handle, cctxt->log_fp);
+                break;
 	case LDTP_CMD_VERIFYPUSHBUTTON:
 		if (is_object_push_button (cctxt->gui_handle->handle, cctxt->log_fp))
 			error = LDTP_ERROR_SUCCESS;
