@@ -742,7 +742,7 @@ get_object_info (Accessible *accessible)
  */
 static char*
 add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
-		 GHashTable *current_context, gboolean overloadedWindowName,
+		 GHashTable *current_context, gboolean overloaded_window_name,
 		 gboolean remap_context)
 {
 	gchar *name = NULL;
@@ -754,14 +754,15 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 	char *accessible_desc = NULL;
 	char *tmp_parent_name = parent_name;
 	gchar *key_binding = NULL;
-	GHashTable *hash_attributes;
+	gchar *obj_hierarchy_name = NULL;
+	GHashTable *hash_attributes = NULL;
 	OBJECT_INFO *cur_obj_info = NULL;
 
 	if (!accessible)
 		return NULL;
 	cur_obj_info = get_object_info (accessible);
 
-	if (overloadedWindowName) {
+	if (overloaded_window_name) {
 		Accessible *parent = Accessible_getParent (accessible);
 		if (parent) {
 			char *name = Accessible_getName (parent);
@@ -848,7 +849,8 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 		}
 	}
 #endif
-	if (label && g_ascii_strcasecmp (label, "") != 0 && overloadedWindowName == FALSE) {
+	obj_hierarchy_name = g_strdup_printf ("%s#%d", cur_obj_info->prefix, cur_obj_info->instance_index);
+	if (label && g_ascii_strcasecmp (label, "") != 0 && overloaded_window_name == FALSE) {
 		char *stripped_data = NULL;
 		char *value = NULL;
 		gboolean flag = FALSE;
@@ -898,7 +900,7 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 				if (name)
 					g_free (name);
 				name = g_strdup_printf ("%s%s%ld", cur_obj_info->prefix, stripped_data, label_index);
-				if (name)
+				if (ldtp_debug && name)
 					g_print ("Name: %s - %d - %d\n", name, flag, remap_context);
 				label_index++;
 				flag = g_hash_table_lookup_extended (current_context, name, NULL, NULL);
@@ -906,23 +908,23 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 		} else if (name && current_context) {
 			GHashTable *context = NULL;
 			while (1) {
-				g_print ("Name: %s\n", name);
+				if (ldtp_debug)
+					g_print ("Name: %s\n", name);
 				if (g_hash_table_lookup_extended (current_context, name, NULL, (gpointer) &context)) {
 					if (context) {
 						char *prop = get_property (context, "parent", NULL);
 						char *remap = get_property (context, "remap", NULL);
 						g_hash_table_foreach (context, (GHFunc)&print_attributes, NULL);
-						if (prop && parent_name)
+						if (ldtp_debug && prop && parent_name)
 							g_print ("remap_context: parent: %s - prop: %s %s %d\n",
 								 parent_name, prop, name, __LINE__);
 						if (!remap && prop && (g_utf8_collate (prop, parent_name) == 0)) {
-							g_print ("remap_context: parent: %s - prop: %s %s %d\n",
-								 parent_name, prop, name, __LINE__);
 							break;
 						}
 					}
 				} else {
-					g_print ("Lookup fails: %s\n", name);
+					if (ldtp_debug)
+						g_print ("Lookup fails: %s\n", name);
 					break;
 				}
 				g_free (name);
@@ -933,7 +935,7 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 		if (ldtp_debug && name && tmp_parent_name)
 			g_print ("Stripped name, prefix, index: %s - Parent: %s\n", name, tmp_parent_name);
 		g_free (stripped_data);
-	} else if (overloadedWindowName) {
+	} else if (overloaded_window_name) {
 		name = g_strdup (accessible_name);
 	} else {
 		name = g_strdup_printf ("%s%d", cur_obj_info->prefix, cur_obj_info->instance_index);
@@ -944,22 +946,26 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 					if (context) {
 						char *prop = get_property (context, "parent", NULL);
 						char *remap = get_property (context, "remap", NULL);
-						if (prop && parent_name)
+						if (ldtp_debug && prop && parent_name)
 							g_print ("remap_context: parent: %s - prop: %s %d\n",
 								 parent_name, prop, __LINE__);
 						if (!remap && prop && (g_utf8_collate (prop, parent_name) == 0)) {
-							g_print ("remap_context: parent: %s - prop: %s %d\n",
-								 parent_name, prop, __LINE__);
 							break;
 						}
 					}
 				} else {
-					g_print ("Lookup fails: %s\n", name);
+					if (ldtp_debug)
+						g_print ("Lookup fails: %s\n", name);
 					break;
 				}
 				g_free (name);
 				cur_obj_info->instance_index++;
-				name = g_strdup_printf ("%s%d", cur_obj_info->prefix, cur_obj_info->instance_index);
+				name = g_strdup_printf ("%s%d", cur_obj_info->prefix,
+							cur_obj_info->instance_index);
+				if (obj_hierarchy_name)
+					g_free (obj_hierarchy_name);
+				obj_hierarchy_name = g_strdup_printf ("%s#%d", cur_obj_info->prefix,
+								      cur_obj_info->instance_index);
 			}
 		}
 	}
@@ -1024,22 +1030,29 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 						     g_strdup (tmp_parent_name));
 			if ((tmp_parent_table = g_hash_table_lookup (current_context, tmp_parent_name)) != NULL) {
 				if (tmp_parent_table) {
-					gchar *childrens = NULL;
-					if (g_hash_table_lookup_extended (tmp_parent_table, "childrens", NULL, (gpointer) &childrens)) {
-						childrens = g_strdup_printf ("%s %s", childrens, name);
-						if (childrens) {
-							g_print ("Childrens: %s\n", childrens);
-							//g_hash_table_remove (tmp_parent_table, "childrens");
-							g_hash_table_replace (tmp_parent_table, g_strdup ("childrens"), childrens);
-						} else
-							g_print ("Unable to create childrens");
+					gchar *children = NULL;
+					if (g_hash_table_lookup_extended (tmp_parent_table, "children", NULL, (gpointer) &children)) {
+						children = g_strdup_printf ("%s %s", children, name);
+						if (children) {
+							if (ldtp_debug)
+							g_print ("Children: %s\n", children);
+							//g_hash_table_remove (tmp_parent_table, "children");
+							g_hash_table_replace (tmp_parent_table, g_strdup ("children"), children);
+						} else {
+							if (ldtp_debug)
+								g_print ("Unable to create children");
+						}
 					} else {
-						g_hash_table_insert (tmp_parent_table, g_strdup ("childrens"), g_strdup (name));
+						g_hash_table_insert (tmp_parent_table, g_strdup ("children"), g_strdup (name));
 					}
-				} else
-					g_print ("Parent table NULL\n");
-			} else
-				g_print ("Unable to find parent\n");
+				} else {
+					if (ldtp_debug)
+						g_print ("Parent table NULL\n");
+				}
+			} else {
+				if (ldtp_debug)
+					g_print ("Unable to find parent\n");
+			}
 			g_hash_table_insert (hash_attributes,
 					     g_strdup ("child_index"),
 					     g_strdup_printf ("%ld", child_index));
@@ -1061,6 +1074,11 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 						     g_strdup ("description"),
 						     g_strdup (accessible_desc));
 			}
+			if (obj_hierarchy_name) {
+				g_hash_table_insert (hash_attributes,
+						     g_strdup ("obj_index"),
+						     obj_hierarchy_name);
+			}
 		}
 		if (current_context) {
 			if (remap_context) {
@@ -1071,7 +1089,8 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 			} else {
 				g_hash_table_insert (current_context, g_strdup (name), hash_attributes);
 			}
-			g_print ("Added: %s to current_context\n", name);
+			if (ldtp_debug)
+				g_print ("Added: %s to current_context\n", name);
 		} else
 			g_hash_table_destroy (hash_attributes);
 		g_free (label);
@@ -1084,7 +1103,7 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 			accessible_name = NULL;
 		}
 
-		if (overloadedWindowName)
+		if (overloaded_window_name)
 			g_free (tmp_parent_name);
 
 		SPI_freeString (accessible_desc);
@@ -1102,7 +1121,7 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 			g_free (accessible_name);
 			accessible_name = NULL;
 		}
-		if (overloadedWindowName)
+		if (overloaded_window_name)
 			g_free (tmp_parent_name);
 
 		SPI_freeString (accessible_desc);
@@ -1117,7 +1136,7 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 static void
 accessible_object_handle (Accessible *accessible, char *parent_name, 
 			  long child_index, GHashTable *current_context,
-			  gboolean overloadedWindowName, gboolean remap_context)
+			  gboolean overloaded_window_name, gboolean remap_context)
 {
 	long i, num_child;
 	Accessible *child;
@@ -1134,12 +1153,11 @@ accessible_object_handle (Accessible *accessible, char *parent_name,
 		if (ldtp_debug)
 			g_print ("Child Index: %ld\n", child_index);
 		temp_parent = add_appmap_data (accessible, parent_name, child_index,
-					       current_context, overloadedWindowName,
+					       current_context, overloaded_window_name,
 					       remap_context);
-		if (overloadedWindowName)
-			overloadedWindowName = FALSE;
-	}
-	else
+		if (overloaded_window_name)
+			overloaded_window_name = FALSE;
+	} else
 		temp_parent = parent_name;
 	if (temp_parent) {
 		if (ldtp_debug)
@@ -1159,7 +1177,7 @@ accessible_object_handle (Accessible *accessible, char *parent_name,
 			*/
 			accessible_object_handle (child, current_parent,
 						  i, current_context,
-						  overloadedWindowName,
+						  overloaded_window_name,
 						  remap_context);
 			Accessible_unref (child);
 		}
@@ -1188,8 +1206,7 @@ get_window_name_in_appmap_format (char *window_name, AccessibleRole role)
 			stripped_data = escape_character (window_name, ' ');
 			if (ldtp_debug && stripped_data)
 				g_print ("Stripped space: %s\n", stripped_data);
-		}
-		else
+		} else
 			stripped_data = g_strdup (window_name);
 		if (ldtp_debug && window_name && stripped_data)
 			g_print ("Before strip %s - After %s\n", window_name, stripped_data);
@@ -1212,11 +1229,11 @@ get_window_name_in_appmap_format (char *window_name, AccessibleRole role)
 
 GHashTable *do_remap (Accessible *accessible_context, char *name, GHashTable *context,
 		      GHashTable *context_attributes, gboolean locale_flag,
-		      gboolean overloadedWindowName, gboolean remap_context)
+		      gboolean overloaded_window_name, gboolean remap_context)
 {
 	GHashTable *current_context = NULL;
 
-	if (!name)
+	if (!name || !accessible_context)
 		return NULL;
 
 	reset_count ();
@@ -1224,33 +1241,65 @@ GHashTable *do_remap (Accessible *accessible_context, char *name, GHashTable *co
 	if (remap_context) {
 		accessible_object_handle (accessible_context, name,
 					  Accessible_getIndexInParent (accessible_context),
-					  context, overloadedWindowName, remap_context);
+					  context, overloaded_window_name, remap_context);
 		return NULL;
 	}
 	current_context = g_hash_table_new_full (&g_str_hash, &g_str_equal, destroy_string, destroy_hashtable);
 	if (context_attributes) {
 		if (ldtp_debug)
-			g_print ("Context Attributes NOT NULL - FLAG: %d\n", overloadedWindowName);
+			g_print ("Context Attributes NOT NULL - FLAG: %d\n", overloaded_window_name);
 		g_hash_table_insert (current_context, g_strdup (name), context_attributes);
 		/*
 		 * NOTE: Passing -1 as parameter signifies that the first entry has
 		 * already been added, so should not do add_appmap_data for this call
 		 */
 		accessible_object_handle (accessible_context, name, -1,
-					  current_context, overloadedWindowName,
+					  current_context, overloaded_window_name,
 					  remap_context);
 	} else {
 		char *role = Accessible_getRoleName (accessible_context);
+		long parent_index = Accessible_getIndexInParent (accessible_context);
 		if (ldtp_debug && name && role)
 			g_print ("Context Attributes NULL - %s - %s - %ld - FLAG: %d\n",
 				 name,
 				 role,
-				 Accessible_getIndexInParent (accessible_context),
-				 overloadedWindowName);
+				 parent_index,
+				 overloaded_window_name);
 		SPI_freeString (role);
+		if (parent_index == -1) {
+			// To over come the bug with Thunderbird !
+			// For all frames, Accessible_getIndexInParent function returns -1
+			long i;
+			Accessible *child;
+			gchar *window_name = NULL;
+			gchar *child_name = NULL;
+			Accessible *parent = Accessible_getParent (accessible_context);
+			if (!parent)
+				return NULL;
+
+			g_print ("*** Overcome the bug Accessible_getIndexInParent returns -1 ***\n");
+
+			window_name = Accessible_getName (accessible_context);
+			for (i = 0; i < Accessible_getChildCount (parent); i++) {
+				child = Accessible_getChildAtIndex (parent, i);
+				if (!child)
+					continue;
+				child_name = Accessible_getName (child);
+				Accessible_unref (child);
+				if (window_name && child_name &&
+				    g_utf8_collate (window_name, child_name) == 0) {
+					SPI_freeString (child_name);
+					parent_index = i;
+					break;
+				}
+				SPI_freeString (child_name);
+			}
+			SPI_freeString (window_name);
+			Accessible_unref (parent);
+		}
 		accessible_object_handle (accessible_context, name,
-					  Accessible_getIndexInParent (accessible_context),
-					  current_context, overloadedWindowName, remap_context);
+					  parent_index, current_context,
+					  overloaded_window_name, remap_context);
 	}
 	return current_context;
 }
