@@ -42,9 +42,9 @@ set_value (Accessible *object, long numvalue, FILE *log_fp)
 	flag = AccessibleValue_setCurrentValue (accessible_value, numvalue);
 	Accessible_unref (accessible_value);
 	if (flag)
-		return  (LDTP_ERROR_SUCCESS);
+		return LDTP_ERROR_SUCCESS;
 	else {
-		error =  (LDTP_ERROR_UNABLE_TO_SET_SPIN_BUTTON_VALUE);
+		error = LDTP_ERROR_UNABLE_TO_SET_SPIN_BUTTON_VALUE;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -71,9 +71,9 @@ verify_set_value (Accessible *object, double value, FILE *log_fp)
 	LDTPErrorCode error;
 
 	if (get_value (object) == value)
-		return  (LDTP_ERROR_SUCCESS);
+		return LDTP_ERROR_SUCCESS;
 	else {
-		error =  (LDTP_ERROR_UNABLE_TO_SPIN_BUTTON_VALUES_NOT_SAME);
+		error = LDTP_ERROR_UNABLE_TO_SPIN_BUTTON_VALUES_NOT_SAME;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -98,9 +98,9 @@ spin_button_main (LDTPClientContext* cctxt, int command)
 		if (cctxt->resp->data)
 			cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
 		if (value)
-			error =  (LDTP_ERROR_SUCCESS);
+			error = LDTP_ERROR_SUCCESS;
 		else
-			error =  (LDTP_ERROR_UNABLE_TO_GET_VALUE);
+			error = LDTP_ERROR_UNABLE_TO_GET_VALUE;
 		break;
 	case LDTP_CMD_VERIFYSETVALUE:
 		if (cctxt->req->arg_list && cctxt->req->arg_list->data)
@@ -109,7 +109,7 @@ spin_button_main (LDTPClientContext* cctxt, int command)
 					  cctxt->log_fp);
 		break;
 	default:
-		error =  (LDTP_ERROR_COMMAND_NOT_IMPLEMENTED);
+		error = LDTP_ERROR_COMMAND_NOT_IMPLEMENTED;
 	}
 	return error;
 }
