@@ -36,7 +36,7 @@ is_object_status_bar (Accessible *object, FILE *log_fp)
 {
 	if (wait_till_object_state_contains (object, STATUS_BAR, log_fp) != 0) {
 		LDTPErrorCode error;
-		error =  (LDTP_ERROR_INVALID_OBJECT_STATE);
+		error = LDTP_ERROR_INVALID_OBJECT_STATE;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return FALSE;
 	}
@@ -77,7 +77,7 @@ get_statusbar_text (Accessible *object, GSList **l, FILE *log_fp)
   
 	if (statusbar_text == NULL) {
 		LDTPErrorCode error;
-		error =  (LDTP_ERROR_STATUSBAR_GETTEXT_FAILED);
+		error = LDTP_ERROR_STATUSBAR_GETTEXT_FAILED;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		Accessible_unref (text);
 		return error;
@@ -85,7 +85,7 @@ get_statusbar_text (Accessible *object, GSList **l, FILE *log_fp)
 	*l = g_slist_prepend (*l, g_strdup (statusbar_text));
 	SPI_freeString (statusbar_text);
 	Accessible_unref (text);
-	return  (LDTP_ERROR_SUCCESS);
+	return LDTP_ERROR_SUCCESS;
 }
 
 LDTPErrorCode
@@ -95,15 +95,15 @@ status_bar_main (LDTPClientContext* cctxt, int command)
 	switch (command) {
 	case LDTP_CMD_VERIFYSTATUSBAR:
 		if (is_object_status_bar (cctxt->gui_handle->handle, cctxt->log_fp))
-			error =  (LDTP_ERROR_SUCCESS);
+			error = LDTP_ERROR_SUCCESS;
 		else
-			error =  (LDTP_ERROR_INVALID_OBJECT_STATE);
+			error = LDTP_ERROR_INVALID_OBJECT_STATE;
 		break;
 	case LDTP_CMD_VERIFYSTATUSBARVISIBLE:
 		if (is_status_bar_state_visible (cctxt->gui_handle->handle))
-			error =  (LDTP_ERROR_SUCCESS);
+			error = LDTP_ERROR_SUCCESS;
 		else
-			error =  (LDTP_ERROR_STATUSBAR_NOT_VISIBLE);
+			error = LDTP_ERROR_STATUSBAR_NOT_VISIBLE;
 		break;
 	case LDTP_CMD_GETSTATUSBARTEXT:
 		error = get_statusbar_text (cctxt->gui_handle->handle, &cctxt->req->arg_list,
@@ -115,7 +115,7 @@ status_bar_main (LDTPClientContext* cctxt, int command)
 		}
 		break;
 	default:
-		error =  (LDTP_ERROR_COMMAND_NOT_IMPLEMENTED);
+		error = LDTP_ERROR_COMMAND_NOT_IMPLEMENTED;
 	}
 	return error;
 }
