@@ -31,6 +31,7 @@
 #include "ldtp-gui-comp.h"
 #include "ldtp-utils.h"
 
+extern gboolean ldtp_debug;
 extern gint ldtp_obj_timeout;
 
 /*
@@ -157,105 +158,288 @@ sort_column (Accessible *object, char *column_header_text, FILE *log_fp)
 }  
 
 static Accessible*
-get_row (Accessible *object, char *row_text, FILE *log_fp)
+get_row (Accessible *object, char *row_text, long *row, long *column, FILE *log_fp)
 {
 	long i, j;
-	long n_rows, n_cols;
+	long n_rows = 0, n_cols = 0;
 	Accessible *cell;
 	AccessibleTable *table;
 	char msg [256];
 	int time_wait = ldtp_obj_timeout;
 	time_t start, cur;
+	time_t start_rows, cur_rows;
 
-	table = Accessible_getTable (object);
-	/*
-	  Copied from LTFX source code
-	*/
-	start = cur = time (NULL);
-
-	if (!ldtp_obj_timeout) {
+	if (ldtp_obj_timeout <= 0) {
 		time_wait = 5; // Max wait for 5 seconds
 	}
-
-	while (difftime (cur, start) < time_wait) {
-		n_rows = AccessibleTable_getNRows (table);
-		if (!n_rows) {
+	start = cur = time (NULL);
+	while (difftime (cur, start) < time_wait * 2) {
+		table = Accessible_getTable (object);
+		if (!table) {
 			ldtp_nsleep (0, 30000);
 
 			cur = time (NULL);
-		} else {
-			break;
+			continue;
 		}
-	}
-	n_cols = AccessibleTable_getNColumns (table);
-	g_print ("Number of rows: %ld\tColumn: %ld\n", n_rows, n_cols);
+		/*
+		  Copied from LTFX source code
+		*/
+		start_rows = cur_rows = time (NULL);
 
-	for (i = 0; i < n_rows; i++) {
-		for (j = 0; j < n_cols; j++) {
-			long child_count;
-			cell = AccessibleTable_getAccessibleAt (table, i, j);
-			child_count = Accessible_getChildCount (cell);
-			g_print ("Child count: %ld\n", child_count);
-			if (child_count > 0) {
-				long k;
-				char *name;
-				Accessible *child;
-				for (k = 0; k < child_count; k++) {
-					child = Accessible_getChildAtIndex (cell, k);
-					name = Accessible_getName (child);
-					g_print ("Table cell name: %s\n", name);
-					if (g_utf8_collate (name, row_text) == 0) {
-						SPI_freeString (name);
-						Accessible_unref (cell);
-						Accessible_unref (table);
-						return child;
-					} // if
-					SPI_freeString (name);
-					Accessible_unref (child);
-				} // for
+		g_print ("Timeout: %d - %d\n", time_wait, ldtp_obj_timeout);
+		while (difftime (cur_rows, start_rows) < time_wait) {
+			n_rows = AccessibleTable_getNRows (table);
+			if (!n_rows) {
+				ldtp_nsleep (0, 30000);
+
+				cur_rows = time (NULL);
 			} else {
-				char *name;
-				name = Accessible_getName (cell);
-				g_print ("Table cell name: %s\n", name);
-				if (g_utf8_collate (name, row_text) == 0) {
-					SPI_freeString (name);
-					Accessible_unref (table);
-					return cell;
-				} // if
-				SPI_freeString (name);
+				break;
 			}
-			Accessible_unref (cell);
-		} // for j
-	} // for i
+		}
+		n_cols = AccessibleTable_getNColumns (table);
+		g_print ("Number of rows: %ld\tColumn: %ld\n", n_rows, n_cols);
+
+		for (i = 0; i < n_rows; i++) {
+			for (j = 0; j < n_cols; j++) {
+				long child_count;
+				cell = AccessibleTable_getAccessibleAt (table, i, j);
+				child_count = Accessible_getChildCount (cell);
+				g_print ("Child count: %ld\n", child_count);
+				if (child_count > 0) {
+					long k;
+					char *name;
+					Accessible *child;
+					for (k = 0; k < child_count; k++) {
+						child = Accessible_getChildAtIndex (cell, k);
+						if (child) {
+							name = Accessible_getName (child);
+							if (name == NULL) {
+								Accessible_unref (child);
+								continue;
+							}
+							if (ldtp_debug) {
+								g_print ("Table cell name: %s\n", name);
+								g_print ("Table cell child count: %ld\n",
+									 Accessible_getChildCount (child));
+							}
+							if (g_utf8_collate (name, row_text) == 0) {
+								SPI_freeString (name);
+								Accessible_unref (cell);
+								Accessible_unref (table);
+								*row = i; *column = j;
+								return child;
+							} // if
+							SPI_freeString (name);
+							Accessible_unref (child);
+						}
+					} // for
+				} else {
+					char *name;
+					name = Accessible_getName (cell);
+					if (name) {
+						g_print ("Table cell name: %s\n", name);
+						if (g_utf8_collate (name, row_text) == 0) {
+							SPI_freeString (name);
+							Accessible_unref (table);
+							*row = i; *column = j;
+							return cell;
+						} // if
+						SPI_freeString (name);
+					}
+				}
+				Accessible_unref (cell);
+			} // for j
+		} // for i
+		ldtp_nsleep (0, 30000);
+
+		cur = time (NULL);
+		Accessible_unref (table);
+	}
 
-	Accessible_unref (table);
 	g_sprintf (msg, "Unable to find %s in table", row_text);
 	log_msg (LDTP_LOG_CAUSE, msg, log_fp);
 	return NULL;
 }
 
+static LDTPErrorCode
+get_cell_value (LDTPClientContext* cctxt)
+{
+	long i, j;
+	long row = 0, column = 0, n_rows, n_cols;
+	int  child_type;
+	long child_count, child_count_level2;
+	gboolean flag = FALSE;
+	GSList *l;
+	LDTPErrorCode error;
+
+	AccessibleTable *table;
+	Accessible *child_object, *child_level2, *child_level3;
+	Accessible *parent_cell;
+	Accessible *object = cctxt->gui_handle->handle;
+
+	l = cctxt->req->arg_list;
+	if (l && l->data) {
+		row = atol (l->data);
+		l = l->next;
+		if (l && l->data)
+			column = atol (l->data);
+	}
+
+	table = Accessible_getTable (object);
+	n_rows = AccessibleTable_getNRows (table);
+	n_cols = AccessibleTable_getNColumns (table);
+	g_print ("Rows: %ld -- Columns: %ld", n_rows, n_cols);
+
+	if (n_rows < row) {
+		Accessible_unref (table);
+		error =  (LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT);
+		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
+		return error;
+	}
+	if (n_cols < column) {
+		Accessible_unref (table);
+		error =  (LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT);
+		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
+		return error;
+	}
+
+	parent_cell = AccessibleTable_getAccessibleAt (table, row, column);
+	child_count = Accessible_getChildCount (parent_cell);
+	child_object = NULL;
+  
+	if (child_count == 0) {
+		flag = TRUE;
+		child_object = parent_cell;
+	}
+	else {
+		for (i = 0; i < child_count && flag == 0; i++) {
+			child_level2 = Accessible_getChildAtIndex (parent_cell, i);
+			child_count_level2 = Accessible_getChildCount (child_level2);
+			if (child_count_level2 > 0) {
+				for (j = 0; j < child_count_level2 && flag == FALSE; j++) {
+					child_level3 = Accessible_getChildAtIndex (child_level2, j);
+					if (Accessible_isText (child_level3)) {
+						flag = TRUE;
+						child_object = child_level3;
+						break;
+					}
+					Accessible_unref (child_level3);
+				}
+			}
+			else {
+				if (Accessible_isText (child_level2)) {
+					flag = TRUE;
+					child_object = child_level2;
+				}
+			}
+			if (flag == FALSE)
+				Accessible_unref (child_level2);
+		}
+		Accessible_unref (parent_cell);
+	}
+	cctxt->gui_handle->handle = child_object;
+	if (flag == TRUE) {
+		if (Accessible_isComponent (child_object)) {
+			child_type = get_object_type (child_object);
+			g_print ("Child type is %d ", child_type);
+			if (child_type == SPI_ROLE_TEXT || child_type == SPI_ROLE_TABLE_CELL) {
+				if (Accessible_isText (child_object)) {
+					g_print ("Table cell is a text object\n");
+					cctxt->req->arg_list = g_slist_prepend (cctxt->req->arg_list,
+										"0");
+					cctxt->req->arg_list = g_slist_prepend (cctxt->req->arg_list,
+										g_strdup_printf ("0"));
+					error = text_main (cctxt, LDTP_CMD_GETTEXTVALUE);
+					cctxt->req->arg_list = g_slist_remove_all (cctxt->req->arg_list, "0");
+					Accessible_unref (table);
+					Accessible_unref (child_object);
+					cctxt->gui_handle->handle = object;
+					return error;
+				}
+			}
+		} //if
+		Accessible_unref (table);
+		if (child_object)
+			Accessible_unref (child_object);
+		cctxt->gui_handle->handle = object;
+	}
+	else {
+		log_msg (LDTP_LOG_CAUSE, "Table cell has no child of type TEXT", cctxt->log_fp);
+		Accessible_unref (table);
+		if (child_object)
+			Accessible_unref (child_object);
+		cctxt->gui_handle->handle = object;
+		error =  (LDTP_ERROR_NO_CHILD_TEXT_TYPE_UNDER_TABLE);
+		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
+		return error;
+	}
+	error =  (LDTP_ERROR_GET_TABLE_CELL_FAILED);
+	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
+	return error;
+}
+
 /*
   Select the row in the given object
 */
 static LDTPErrorCode
-select_row (Accessible *object, char *row_text, FILE *log_fp)
+select_row (LDTPClientContext *cctxt, Accessible *object, char *row_text, FILE *log_fp)
 {
 	Accessible *cell;
 	LDTPErrorCode error;
+	int time_wait = ldtp_obj_timeout;
+	time_t start, cur;
+	long row = -1;
+	long column = -1;
 
-	cell = get_row (object, row_text, log_fp);
+	if (!object) {
+		g_print ("%s - %d\n", __FILE__, __LINE__);
+		error =  (LDTP_ERROR_UNABLE_TO_SELECT_ROW);
+		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
+		return error;
+	}
+
+	if (ldtp_obj_timeout <= 0) {
+		time_wait = 5; // Max wait for 5 seconds
+	}
+
+	cell = get_row (object, row_text, &row, &column, log_fp);
 	if (cell) {
 		if (Accessible_isComponent (cell)) {
 			SPIBoolean flag;
 			AccessibleComponent *accessible_component;
-			accessible_component = Accessible_getComponent (cell);
-			flag = AccessibleComponent_grabFocus (accessible_component);
-			Accessible_unref (accessible_component);
+			start = cur = time (NULL);
+			while (difftime (cur, start) < time_wait) {
+				accessible_component = Accessible_getComponent (cell);
+				if (accessible_component) {
+					flag = AccessibleComponent_grabFocus (accessible_component);
+					Accessible_unref (accessible_component);
+				}
+				if (flag) {
+					if (Accessible_isText (cell)) {
+						AccessibleText *text = Accessible_getText (cell);
+						if (text) {
+							char *name = AccessibleText_getText (text, 0, LONG_MAX);
+							if (name) {
+								g_print ("Cell content: %s\n", name);
+								SPI_freeString (name);
+							}
+							Accessible_unref (text);
+						}
+					}
+					break;
+				}
+				ldtp_nsleep (0, 30000);
+
+				cur = time (NULL);
+			}
 			Accessible_unref (cell);
-			if (flag)
+			if (flag) {
+				g_print ("Grabed focus\n");
 				return  (LDTP_ERROR_SUCCESS);
-			else {
-				error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+			} else {
+				g_print ("Unable to grab focus\n");
+				error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 				return error;
 			}
@@ -327,7 +511,10 @@ double_click_row (Accessible *object, char *row_text, FILE *log_fp)
 	Accessible *cell;
 	AccessibleComponent *component;
 
-	cell = get_row (object, row_text, log_fp);
+	long row = -1;
+	long column = -1;
+
+	cell = get_row (object, row_text, &row, &column, log_fp);
 	if (cell) {
 		if (Accessible_isComponent (cell))
 			component = Accessible_getComponent (cell);
@@ -581,7 +768,7 @@ verify_cell_value (LDTPClientContext* cctxt)
 	g_print ("Table - number of rows:%ld\tColumn: %ld\n", n_rows, n_cols);
 	child_object = AccessibleTable_getAccessibleAt (table, row, column);
 
-	do { 
+	do {
 		child_count = Accessible_getChildCount (child_object);
 		if (child_count > 0) {
 			Accessible *tmp_child;
@@ -594,7 +781,7 @@ verify_cell_value (LDTPClientContext* cctxt)
 	cctxt->gui_handle->handle = child_object;
 	if (Accessible_isComponent (child_object)) {
 		child_type = get_object_type (child_object);
-		g_print ("Child type is %d ", child_type);
+		g_print ("Child type is %d\n", child_type);
 		if (child_type == SPI_ROLE_TABLE_CELL) {
 			action = Accessible_getAction (child_object);
 			n_actions = AccessibleAction_getNActions (action);
@@ -734,8 +921,8 @@ set_cell_value (LDTPClientContext* cctxt)
 	g_print ("Table - number of rows:%ld\tColumn: %ld\n", n_rows, n_cols);
 
 	cell = AccessibleTable_getAccessibleAt (table, row, column);
-	cell = cctxt->gui_handle->handle;
-	if (Accessible_isComponent (cell)) {
+	cctxt->gui_handle->handle = cell;
+	if (cell && Accessible_isComponent (cell)) {
 		long j;
 		action = Accessible_getAction (cell);
 		n_actions = AccessibleAction_getNActions (action);
@@ -758,7 +945,7 @@ set_cell_value (LDTPClientContext* cctxt)
 			if (g_ascii_strcasecmp (name, "edit") == 0) {
 				GSList *l = g_slist_last (cctxt->req->arg_list);
 				cctxt->req->arg_list = g_slist_prepend (cctxt->req->arg_list, l->data);
-				g_print ("Table cell is text box\n");
+				g_print ("Table cell is text box %s - %d\n", __FILE__, __LINE__);
 				error = text_main (cctxt, LDTP_CMD_SETTEXTVALUE);
 				cctxt->req->arg_list = g_slist_remove (cctxt->req->arg_list, l->data);
 				SPI_freeString (name);
@@ -773,7 +960,8 @@ set_cell_value (LDTPClientContext* cctxt)
 		Accessible_unref (action);
 	} //if
 	Accessible_unref (table);
-	Accessible_unref (cell);
+	if (cell)
+		Accessible_unref (cell);
 	cctxt->gui_handle->handle = object;
 	error =  (LDTP_ERROR_SET_TABLE_CELL_FAILED);
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
@@ -1139,123 +1327,6 @@ verify_partial_cell_value (LDTPClientContext* cctxt)
 	return error;
 }
 
-static LDTPErrorCode
-get_cell_value (LDTPClientContext* cctxt)
-{
-	long i, j;
-	long row = 0, column = 0, n_rows, n_cols;
-	int  child_type;
-	long child_count, child_count_level2;
-	gboolean flag = FALSE;
-	GSList *l;
-	LDTPErrorCode error;
-
-	AccessibleTable *table;
-	Accessible *child_object, *child_level2, *child_level3;
-	Accessible *parent_cell;
-	Accessible *object = cctxt->gui_handle->handle;
-
-	l = cctxt->req->arg_list;
-	if (l && l->data) {
-		row = atol (l->data);
-		l = l->next;
-		if (l && l->data)
-			column = atol (l->data);
-	}
-
-	table = Accessible_getTable (object);
-	n_rows = AccessibleTable_getNRows (table);
-	n_cols = AccessibleTable_getNColumns (table);
-	g_print ("Rows: %ld -- Columns: %ld", n_rows, n_cols);
-
-	if (n_rows < row) {
-		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT);
-		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
-		return error;
-	}
-	if (n_cols < column) {
-		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT);
-		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
-		return error;
-	}
-
-	parent_cell = AccessibleTable_getAccessibleAt (table, row, column);
-	child_count = Accessible_getChildCount (parent_cell);
-	child_object = NULL;
-  
-	if (child_count == 0) {
-		flag = TRUE;
-		child_object = parent_cell;
-	}
-	else {
-		for (i = 0; i < child_count && flag == 0; i++) {
-			child_level2 = Accessible_getChildAtIndex (parent_cell, i);
-			child_count_level2 = Accessible_getChildCount (child_level2);
-			if (child_count_level2 > 0) {
-				for (j = 0; j < child_count_level2 && flag == 0; j++) {
-					child_level3 = Accessible_getChildAtIndex (child_level2, j);
-					if (Accessible_isText (child_level3)) {
-						flag = TRUE;
-						child_object = child_level3;
-						break;
-					}
-					Accessible_unref (child_level3);
-				}
-			}
-			else {
-				if (Accessible_isText (child_level2)) {
-					flag = TRUE;
-					child_object = child_level2;
-				}
-			}
-			if (flag == FALSE)
-				Accessible_unref (child_level2);
-		}
-		Accessible_unref (parent_cell);
-	}
-	cctxt->gui_handle->handle = child_object;
-	if (flag == TRUE) {
-		if (Accessible_isComponent (child_object)) {
-			child_type = get_object_type (child_object);
-			g_print ("Child type is %d ", child_type);
-			if (child_type == SPI_ROLE_TEXT || child_type == SPI_ROLE_TABLE_CELL) {
-				if (Accessible_isText (child_object)) {
-					g_print ("Table cell is a text object\n");
-					cctxt->req->arg_list = g_slist_prepend (cctxt->req->arg_list,
-										"0");
-					cctxt->req->arg_list = g_slist_prepend (cctxt->req->arg_list,
-										g_strdup_printf ("0"));
-					error = text_main (cctxt, LDTP_CMD_GETTEXTVALUE);
-					cctxt->req->arg_list = g_slist_remove_all (cctxt->req->arg_list, "0");
-					Accessible_unref (table);
-					Accessible_unref (child_object);
-					cctxt->gui_handle->handle = object;
-					return error;
-				}
-			}
-		} //if
-		Accessible_unref (table);
-		if (child_object)
-			Accessible_unref (child_object);
-		cctxt->gui_handle->handle = object;
-	}
-	else {
-		log_msg (LDTP_LOG_CAUSE, "Table cell has no child of type TEXT", cctxt->log_fp);
-		Accessible_unref (table);
-		if (child_object)
-			Accessible_unref (child_object);
-		cctxt->gui_handle->handle = object;
-		error =  (LDTP_ERROR_NO_CHILD_TEXT_TYPE_UNDER_TABLE);
-		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
-		return error;
-	}
-	error =  (LDTP_ERROR_GET_TABLE_CELL_FAILED);
-	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
-	return error;
-}
-
 static LDTPErrorCode
 get_row_index (Accessible *object, GSList **l, FILE *log_fp)
 {
@@ -1334,7 +1405,10 @@ single_click_row (Accessible *object, char *row_text, FILE *log_fp)
 	Accessible *cell;
 	AccessibleComponent *component;
 
-	cell = get_row (object, row_text, log_fp);
+	long row = -1;
+	long column = -1;
+
+	cell = get_row (object, row_text, &row, &column, log_fp);
 	if (cell) {
 		if (Accessible_isComponent (cell))
 			component = Accessible_getComponent (cell);
@@ -1383,7 +1457,7 @@ table_main (LDTPClientContext* cctxt, int command)
 		break;
 	case LDTP_CMD_SELECTROW:
 		g_print ("Select row: %s\n", (char *)cctxt->req->arg_list->data);
-		error = select_row (cctxt->gui_handle->handle,
+		error = select_row (cctxt, cctxt->gui_handle->handle,
 				    (char *)cctxt->req->arg_list->data, cctxt->log_fp);
 		break;
 	case LDTP_CMD_DOUBLECLICK:
@@ -1403,6 +1477,26 @@ table_main (LDTPClientContext* cctxt, int command)
 	case LDTP_CMD_SETCELLVALUE:
 		error = set_cell_value (cctxt);
 		break;
+	case LDTP_CMD_SETTEXTVALUE: {
+		if (Accessible_isText (cctxt->gui_handle->handle)) {
+			Accessible *tmp_object = cctxt->gui_handle->handle;
+			cctxt->gui_handle->handle = Accessible_getText (tmp_object);
+			if (cctxt->gui_handle->handle) {
+				error = text_main (cctxt, command);
+				Accessible_unref (cctxt->gui_handle->handle);
+			} else {
+				g_print ("%s - %d\n", __FILE__, __LINE__);
+				error = LDTP_ERROR_UNABLE_TO_SET_TEXT;
+				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
+			}
+			cctxt->gui_handle->handle = tmp_object;
+		} else {
+			g_print ("%s - %d\n", __FILE__, __LINE__);
+			error = LDTP_ERROR_UNABLE_TO_SET_TEXT;
+			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
+		}
+		break;
+	}
 	case LDTP_CMD_GETROWCOUNT:
 		error = get_row_count (cctxt->gui_handle->handle, &cctxt->req->arg_list);
 		if (error == LDTP_ERROR_SUCCESS) {
@@ -1493,7 +1587,7 @@ table_main (LDTPClientContext* cctxt, int command)
 		error = device_main (cctxt, command);
 		break;
 	default:
-		error =  (LDTP_ERROR_COMMAND_NOT_IMPLEMENTED);
+		error = LDTP_ERROR_COMMAND_NOT_IMPLEMENTED;
 	}
 	return error;
 }
